/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd;

import java.io.File;

import org.apache.commons.cli.CommandLine;
import org.apache.log4j.Logger;
import org.openrdf.rio.RDFFormat;

/**
 * In-memory representation of the command line configuration.
 * 
 * @author MSchmidt
 */
public class Config {
	
	private static Logger logger = Logger.getLogger(Config.class);
	
	private static Integer LIMIT_DEFAULT = 3;
	private static OutputType OUTPUT_TYPE_DEFAULT = OutputType.ASK;
	private static RDFFormat RDF_FORMAT_DEFAULT = RDFFormat.RDFXML;
	
	/**
	 * Output file to write the result (if not specified, STDOUT is used)
	 */
	private File outputFile = null;
	
	/**
	 * The RDD input file
	 */
	private File rddFile = null;
	
	/**
	 * The RDF data file
	 */
	private File databaseFile = null;
	
	/**
	 * The RDF data file format (as MIME type)
	 */
	private RDFFormat databaseFileFormat = null;
	
	/**
	 * The output type specified by the user.
	 */
	private OutputType outputType = OUTPUT_TYPE_DEFAULT;
	
	/**
	 * The error file specified by the user.
	 */
	private File errorFile;
	
	/**
	 * The statistics file specified by the user.
	 */
	private File statsFile;
	
	/**
	 * The limit for witnesses shown in evaluation mode (no effect otherwise)
	 */
	private Integer limit = LIMIT_DEFAULT;
	
	public enum OutputType {
		FOL,
		ASK, 
		SELECT
	}

	/**
	 * Constructor initializing the {@link Config} object from command line
	 * @param cmdLine
	 */
	public Config(CommandLine cmdLine) {
		
		try {
			if (cmdLine.hasOption('t')) {
				outputType = OutputType.valueOf(cmdLine.getOptionValue('t'));
			}
			
			if (cmdLine.hasOption('o')) {
				String fileName = cmdLine.getOptionValue('o');
				outputFile = new File(fileName);
			}

			if (cmdLine.hasOption('e')) {
				String fileName = cmdLine.getOptionValue('e');
				errorFile = new File(fileName);
			}
			
			if (cmdLine.hasOption('l')) {
				try {
					String limitStr = cmdLine.getOptionValue('l');
					limit = Integer.valueOf(limitStr);
				} catch (Exception e) {
					logger.warn("Illegal limit specification (must be an integer value): " + e.getMessage());
				}
			}

			if (cmdLine.hasOption('s')) {
				String fileName = cmdLine.getOptionValue('s');
				statsFile = new File(fileName);
			}

			if (cmdLine.hasOption('d')) {
				String databaseFileName = cmdLine.getOptionValue('d');
				databaseFile = new File(databaseFileName);
				if (cmdLine.hasOption('m')) {
					databaseFileFormat =
						RDFFormat.forMIMEType(databaseFileName, RDFFormat.forMIMEType(cmdLine.getOptionValue('m')));
				} else {
					databaseFileFormat =
						RDFFormat.forFileName(databaseFileName, RDF_FORMAT_DEFAULT);
				}
			}
			
			// this parameter is mandatory, so it must be specified
			String rddFileName = cmdLine.getOptionValue('r');
			rddFile = new File(rddFileName);
			if (!rddFile.exists())
				throw new IllegalArgumentException("RDD file '" + rddFileName + "' does not exist.");
			
		} catch (Exception e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	}
	
	public OutputType getOutputType() {
		return outputType;
	}
	
	public File getRddFile() {
		return rddFile;
	}
	
	public File getOutputFile() {
		return outputFile;
	}
	
	public File getDatabaseFile() {
		return databaseFile;
	}
	
	public RDFFormat getDatabaseFileFormat() {
		return databaseFileFormat;
	}
	
	public boolean isEvaluationMode() {
		return databaseFile!=null;
	}
	
	public File getErrorFile() {
		return errorFile;
	}
	
	public File getStatsFile() {
		return statsFile;
	}	
	
	public Integer getLimit() {
		return limit;
	}
	
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("Config(");
		buf.append("outputType=" + outputType + ", ");
		buf.append("rddFile=" + rddFile + ", ");
		buf.append("outputFile=" + outputFile);
		buf.append("errorFile=" + errorFile);
		buf.append("limit=" + limit);
		buf.append("databaseFile=" + databaseFile);
		return buf.toString();
	}
}