/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.fol;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import edu.freiburg.dbis.rdd.fol.FORangeType.Relation;
import edu.freiburg.dbis.rdd.model.IRIWithRangeType;
import edu.freiburg.dbis.rdd.model.Key;
import edu.freiburg.dbis.rdd.model.RangeType;
import edu.freiburg.dbis.rdd.model.RangeTypeBNode;
import edu.freiburg.dbis.rdd.model.RangeTypeIRI;
import edu.freiburg.dbis.rdd.model.RangeTypeLiteral;
import edu.freiburg.dbis.rdd.model.RangeTypeResource;
import edu.freiburg.dbis.rdd.provenance.ConstraintProvenanceInfo;
import edu.freiburg.dbis.rdd.semantics.Environment;
import edu.freiburg.dbis.rdd.sparql.IRI;
import edu.freiburg.dbis.rdd.sparql.SPARQLVariable;
import edu.freiburg.dbis.rdd.sparql.SPARQLVariableFactory;

/**
 * Class for building FO constraints for the individual constraint types.
 * This class implements the core of the RDD semantics.
 * 
 * @author MSchmidt
 */
public class FOConstraintBuilder {

	private static IRI RDF_TYPE = 
		IRI.fromSparqlIri("<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>");
	
	public static SPARQLVariableFactory vf = SPARQLVariableFactory.getInstance();

	/**
	 * - Qualified: (clazz!=null): forall s, o (T(s, prop, o) -> T(s, rdf:type, domain)))
	 * - Unqualified (clazz==null): forall s (T(s, rdf:type, clazz), T(s, prop, o) -> T(s, rdf:type, domain)))
	 **/
	public static List<FOFormula> domainConstraint(
			IRI clazz, IRI prop, IRI domain, ConstraintProvenanceInfo provenanceInfo) {
		
		vf.reset();

		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		SPARQLVariable varO = vf.freshOVariable();
		vars.add(varS);
		vars.add(varO);
		
		FOFormula premise = null;
		if (clazz!=null) {
			FOTriple c1 = new FOTriple(varS, RDF_TYPE, clazz);
			FOTriple c2 = new FOTriple(varS, prop, varO);
			premise = new FOAnd(c1,c2);
		} else {
			premise = new FOTriple(varS, prop, varO);
		}

		FOTriple conclusion =
			new FOTriple(varS, RDF_TYPE, domain);
		
		return convertToListAndSetProvenance(
				new FOForall(vars, new FOImplies(premise, conclusion)),
				provenanceInfo);
	}
	
	
	/**
	 * - Qualified: (clazz!=null): forall s, o (T(s, prop, o) -> T(o, rdf:type, range)))
	 * - Unqualified (clazz==null): forall s (T(s, rdf:type, clazz), T(s, prop, o) -> T(o, rdf:type, range)))
	 **/
	public static List<FOFormula> rangeConstraint(
			IRI clazz, IRI prop, IRI range, ConstraintProvenanceInfo provenanceInfo) {

		vf.reset();

		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		SPARQLVariable varO = vf.freshOVariable();
		vars.add(varS);
		vars.add(varO);
		
		FOFormula premise = null;
		if (clazz!=null) {
			FOTriple c1 = new FOTriple(varS, RDF_TYPE, clazz);
			FOTriple c2 = new FOTriple(varS, prop, varO);
			premise = new FOAnd(c1,c2);
		} else {
			premise = new FOTriple(varS, prop, varO);
		}

		FOTriple conclusion =
			new FOTriple(varO, RDF_TYPE, range);
		
		return convertToListAndSetProvenance(
				new FOForall(vars, new FOImplies(premise, conclusion)),
				provenanceInfo);
	}

	/**
	 * - Qualified: (clazz!=null): forall s, o_1, ..., o_n+1 (T(s,p,o_1), ..., T(s,p,o_n+1) -> someEqual(o_1,...,o_n+1))
	 * - Unqualified (clazz==null): forall s, o_1, ..., o_n+1 (T(s,rdf:type,clazz), T(s,p,o_1), ..., T(s,p,o_n+1) -> someEqual(o_1,...,o_n+1)) 
	 * 
	 * Note that, by grammar, the number n is greater than zero, which is what the method assumes.
	 **/
	public static List<FOFormula> maxConstraint(
			IRI clazz, IRI prop, int n, ConstraintProvenanceInfo provenanceInfo) {
		
		vf.reset();

		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		List<SPARQLVariable> varOs = vf.freshOVariables(n+1);
		vars.add(varS);
		vars.addAll(varOs);

		// premise
		List<FOFormula> conjunction = new LinkedList<FOFormula>();
		if (clazz!=null) {
			conjunction.add(new FOTriple(varS, RDF_TYPE, clazz));
		}
		for (int i=0; i<varOs.size(); i++) {
			conjunction.add(new FOTriple(varS, prop, varOs.get(i)));
		}
		FOAnd premise = new FOAnd(conjunction);

		// build complete formula
		// note: we can safely call someEq here, since nr>=1 (by grammar),
		//       so it follows that varsO>=2
		return convertToListAndSetProvenance(
				new FOForall(vars, new FOImplies(premise, someEqual(varOs))),
				provenanceInfo);
	}

	/**
	 * - Qualified: (clazz!=null): forall s (Resource(s) -> exists o_1, ..., o_n T(s,p,o_1), ..., T(s,p,o_n), allDist(o_1,...,o_n))
	 * - Unqualified (clazz==null): forall s (T(s,rdf:type,clazz) -> exists o_1, ..., o_n T(s,p,o_1), ..., T(s,p,o_n), allDist(o_1,...,o_n)) 
	 * 
	 * Note that, by grammar, the number n is greater than zero, which is what the method assumes.
	 **/
	public static List<FOFormula> minConstraint(
			IRI clazz, IRI prop, int n, ConstraintProvenanceInfo provenanceInfo) {
		
		vf.reset();

		List<SPARQLVariable> forallVars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		forallVars.add(varS);

		List<SPARQLVariable> existsVars = vf.freshOVariables(n);
		
		FOFormula premise = clazz==null ?
			new FORangeType(new FOIRIOrSPARQLVariable(varS), Relation.RESOURCE) :
			new FOTriple(varS, RDF_TYPE, clazz);

		List<FOFormula> conjunction = new LinkedList<FOFormula>();
		for (int i=0; i<existsVars.size(); i++) {
			conjunction.add(new FOTriple(varS, prop, existsVars.get(i)));
		}
		if (existsVars.size()>1)
			conjunction.addAll(allDist(existsVars));
		
		FOFormula innerExists = conjunction.size()>1 ?
				new FOAnd(conjunction) : conjunction.get(0); // conjunction.size()>=1

		return convertToListAndSetProvenance(
				new FOForall(forallVars, new FOImplies(premise, new FOExists(existsVars, innerExists))),
				provenanceInfo);
	}

	/**
	 * The partiality constraint is syntactic sugar for MAX 1.
	 **/
	public static List<FOFormula> partialityConstraint(
			IRI clazz, IRI prop, ConstraintProvenanceInfo provenanceInfo) {
		return maxConstraint(clazz, prop, 1, provenanceInfo);
	}

	/**
	 * The partiality constraint is syntactic sugar MIN 1 & MAX 1.
	 **/
	public static List<FOFormula> totalityConstraint(
			IRI clazz, IRI prop, ConstraintProvenanceInfo provenanceInfo) {
		List<FOFormula> ret = minConstraint(clazz, prop, 1, provenanceInfo);
		ret.addAll(maxConstraint(clazz, prop, 1, provenanceInfo));
		return ret;
	}

	
	/**
	 * - Qualified: (clazz!=null): 	forall s, o(T(s,p,o) -> exists o_1, o_n-1(T(s, q_1, o_1),  T(o_1, q_2, o_2), ..., T(o_n-1,q_n,o)))
	 * - Unqualified: (clazz==null): forall s, o(T(s,rdf:type,clazz), T(s,p,o) -> exists o_1, o_n-1(T(s, q_1, o_1),  T(o_1, q_2, o_2), ..., T(o_n-1,q_n,o))) 
	 * 
	 * Note that, by grammar, the path can never be empty (which is what the method assumes).
	 */
	public static List<FOFormula> pathConstraint(
			IRI clazz, IRI prop, List<IRI> path, ConstraintProvenanceInfo provenanceInfo) {
		
		vf.reset();
		
		List<SPARQLVariable> forallVars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		SPARQLVariable varO = vf.freshOVariable();
		forallVars.add(varS);
		forallVars.add(varO);
		
		List<SPARQLVariable> existsVars = vf.freshOVariables(path.size()-1);

		// premise
		List<FOFormula> premiseConjunction = new LinkedList<FOFormula>();
		if (clazz!=null) {
			premiseConjunction.add(new FOTriple(varS, RDF_TYPE, clazz));
		}
		premiseConjunction.add(new FOTriple(varS, prop, varO));
		FOFormula premise = premiseConjunction.size() == 1 ?
			premiseConjunction.get(0) : new FOAnd(premiseConjunction);
				
		// conclusion
		FOFormula conclusion = null;
		if (existsVars.isEmpty()) { // implies that path.size()==1
			conclusion = new FOTriple(varS,path.get(0),varO);
		} else {
			List<FOFormula> conclusionConjunction = new LinkedList<FOFormula>();
			conclusionConjunction.add(new FOTriple(varS,path.get(0),existsVars.get(0)));
			for (int i=0; i<existsVars.size(); i++) {
				conclusionConjunction.add(
					new FOTriple(
						existsVars.get(i),
						path.get(i+1),
						i+1<existsVars.size() ? existsVars.get(i+1) : varO));
			}
			conclusion = new FOExists(existsVars, new FOAnd(conclusionConjunction));
		}
		
		return convertToListAndSetProvenance(
				new FOForall(forallVars,new FOImplies(premise,conclusion)),
				provenanceInfo);
	}

	/**
	 * Range type constraint, covered through dedicated relations in FOL.
	 */
	public static List<FOFormula> rangeTypeConstraint(
			IRI clazz, IRIWithRangeType propWithRangeType, ConstraintProvenanceInfo provenanceInfo) {
		
		vf.reset();
		
		IRI prop = propWithRangeType.getIRI();

		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		SPARQLVariable varO = vf.freshOVariable();
		vars.add(varS);
		vars.add(varO);
		
		// set up premise
		List<FOFormula> premiseConjunction = new LinkedList<FOFormula>();
		if (clazz!=null) {
			premiseConjunction.add(new FOTriple(varS, RDF_TYPE, clazz));
		}
		premiseConjunction.add(new FOTriple(varS, prop, varO));
		FOFormula premise = premiseConjunction.size() == 1 ?
			premiseConjunction.get(0) : new FOAnd(premiseConjunction);		

		// set up conclusions
		RangeType rt = propWithRangeType.getRangeType();
		Class<?> clazzType = rt==null ? null : rt.getClass();
		
		FOFormula conclusion = null;
		if (clazzType==null) {
			// nothing to be done
		} else if (clazzType.equals(RangeTypeIRI.class)) {
			conclusion = new FORangeType(new FOIRIOrSPARQLVariable(varO), Relation.IRI);
		} else if (clazzType.equals(RangeTypeBNode.class)) {
			conclusion = new FORangeType(new FOIRIOrSPARQLVariable(varO), Relation.BNODE);			
		} else if (clazzType.equals(RangeTypeResource.class)) {
			conclusion = new FORangeType(new FOIRIOrSPARQLVariable(varO), Relation.RESOURCE);
		} else if (clazzType.equals(RangeTypeLiteral.class)) {
			
			FOIRIOrSPARQLVariable wrappedVar = new FOIRIOrSPARQLVariable(varO);
			FOFormula conclusion1 = new FORangeType(wrappedVar, Relation.LITERAL);

			FOFormula conclusion2 = null;
			RangeTypeLiteral rtl = (RangeTypeLiteral)rt;
			if (rtl.getDatatype()!=null)
				conclusion2 = new FODatatype(wrappedVar, rtl.getDatatype());
			
			conclusion = conclusion2==null ? conclusion1 : new FOAnd(conclusion1, conclusion2);
			
		} else {
			// no match: should not occur
			throw new IllegalArgumentException(
					"Invalid type, something's wrong here, "
					+ "the following class is not defined: " 
					+ clazzType.getName());
		}
		
		return convertToListAndSetProvenance(
				new FOForall(vars,new FOImplies(premise, conclusion)),
				provenanceInfo);
	}
	
	/**
	 * Unqualified: induces one constraint, recursively for every subproperty p_s 
	 *   (clazz==null): forall s, o (T(s, p_s, o) -> T(s, p, o))
	 * Qualified: induces one constraint, recursively for every subproperty p_s
	 *   (clazz!=null): forall s, o (T(s,rdf:type,clazz), T(s, p_s, o) -> T(s, p, o))
	 */	
	public static List<FOFormula> subPropertyConstraint(
			IRI clazz, IRI prop, Collection<IRI> subProps, Environment e, ConstraintProvenanceInfo provenanceInfo) {
		return subPropertyConstraint(clazz, prop, subProps, e, new HashSet<IRI>(), provenanceInfo);
	}

	/**
	 * Helper method for the <tt>subPropertyConstraint</tt>, which remembers
	 * recursively & non-recursively visited properties in order to avoid
	 * endless recursion in case of cycles in the subproperty hierarchy.
	 */
	protected static List<FOFormula> subPropertyConstraint(
			IRI clazz, IRI prop, Collection<IRI> subProps, Environment e, 
			Set<IRI> handled, ConstraintProvenanceInfo provenanceInfo) {
		
		List<FOFormula> ret = new LinkedList<FOFormula>();

		// add the constraint for direct subproperties
		for (IRI subProp : subProps) {
			// we skip the sub-property if it (i) has been handled already (to break cycles)
			// or if (ii) it equals to the property itself (to avoid a tautology constraint)
			if (!handled.contains(subProp) && !prop.equals(subProp)) {
				ret.add(singleSubPropertyConstraint(clazz, prop, subProp, provenanceInfo));
				handled.add(subProp);
				
				ret.addAll(
					subPropertyConstraint(clazz,prop,e.P(subProp),e,handled,provenanceInfo));
			}
		}
		
		return ret;
		
	}	
	
	/*
	 * Unqualified: (clazz==null): forall s, o (T(s, p_s, o) -> T(s, p, o))
	 * Qualified: (clazz!=null): forall s, o (T(s,rdf:type,clazz), T(s, p_s, o) -> T(s, p, o))
     */
	protected static FOFormula singleSubPropertyConstraint(
			IRI clazz, IRI prop, IRI subProp, ConstraintProvenanceInfo provenanceInfo) {
		vf.reset();
		
		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		SPARQLVariable varO = vf.freshOVariable();
		vars.add(varS);
		vars.add(varO);
		
		List<FOFormula> premiseConjunction = new LinkedList<FOFormula>();
		if (clazz!=null) {
			premiseConjunction.add(new FOTriple(varS, RDF_TYPE, clazz));
		}
		premiseConjunction.add(new FOTriple(varS, subProp, varO));
		FOFormula premise = premiseConjunction.size() == 1 ?
			premiseConjunction.get(0) : new FOAnd(premiseConjunction);		
		
		FOFormula conclusion = new FOTriple(varS, prop, varO);
		
		FOFormula ret = new FOForall(vars,new FOImplies(premise, conclusion));
		ret.setProvenanceInfo(provenanceInfo);
		return ret;
	}
	
	/**
	 * Only the properties listed in the PROPERTIES sections are allowed
	 * to occur in the dataset.
	 * 
	 * Unqualified: forall s, p, o (T(s,p,o) -> p=p_1 || ... || p=p_n)
	 * Qualified: forall s, p, o (T(s,rdf:type,clazz), T(s,p,o) -> p=p_1 || ... || p=p_n)
	 */
	public static List<FOFormula> propertyCwaConstraint(
			IRI clazz, Collection<IRI> props, ConstraintProvenanceInfo provenanceInfo) {
		
		vf.reset();
		
		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		SPARQLVariable varP = vf.freshPVariable();
		SPARQLVariable varO = vf.freshOVariable();
		vars.add(varS);
		vars.add(varP);
		vars.add(varO);

		List<FOFormula> premiseConjunction = new LinkedList<FOFormula>();
		if (clazz!=null) {
			premiseConjunction.add(new FOTriple(varS, RDF_TYPE, clazz));
		}
		premiseConjunction.add(new FOTriple(varS, varP, varO));
		FOFormula premise = premiseConjunction.size() == 1 ?
			premiseConjunction.get(0) : new FOAnd(premiseConjunction);		
		
		// build conclusion
		List<FOFormula> disjunction = new LinkedList<FOFormula>();
		disjunction.add(
				new FOEquals(
					new FOIRIOrSPARQLVariable(varP), 
					new FOIRIOrSPARQLVariable(RDF_TYPE)));
		for (IRI prop : props) {
			disjunction.add(
				new FOEquals(
					new FOIRIOrSPARQLVariable(varP), 
					new FOIRIOrSPARQLVariable(prop)));
		}
		FOFormula conclusion = new FOOr(disjunction);
		
		return convertToListAndSetProvenance(
				new FOForall(vars,new FOImplies(premise, conclusion)),
				provenanceInfo);

	}
	
	/**
	 * Only the classes listed in are allowed for typing.
	 * 
	 * Formula: forall s, o (T(s,rdf:type,o) -> s=c_1 || ... || s=c_n)
	 */
	public static List<FOFormula> classCwaConstraint(
			Collection<IRI> clazzez, ConstraintProvenanceInfo provenanceInfo) {
		
		vf.reset();

		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS = vf.freshSVariable();
		SPARQLVariable varO = vf.freshOVariable();
		vars.add(varS);
		vars.add(varO);

		FOFormula premise = new FOTriple(varS, RDF_TYPE, varO);
		
		// build conclusion
		List<FOFormula> disjunction = new LinkedList<FOFormula>();
		for (IRI clazz : clazzez) {
			disjunction.add(
				new FOEquals(
					new FOIRIOrSPARQLVariable(varO), 
					new FOIRIOrSPARQLVariable(clazz)));
		}
		FOFormula conclusion = disjunction.isEmpty() ?
			new FOFalse() : new FOOr(disjunction);
		
		return convertToListAndSetProvenance(
			new FOForall(vars,new FOImplies(premise, conclusion)),
				provenanceInfo);
	}

	/**
	 * Formula: forall s1, s2 (T(s1, rdf:type, C), T(s2, rdf:type, C) -> s1 = s2)
	 */
	public static List<FOFormula> classSingletonConstraint(
			IRI clazz, ConstraintProvenanceInfo provenanceInfo) {
		
		List<FOFormula> ret = new ArrayList<FOFormula>();

		// constraint 1: there must exist an instance
		vf.reset();

		SPARQLVariable varExists = vf.freshSVariable();
		List<SPARQLVariable> varsExists = new LinkedList<SPARQLVariable>();
		varsExists.add(varExists);
		FOExists min1 = new FOExists(varsExists, new FOTriple(varExists, RDF_TYPE, clazz));
		min1.setProvenanceInfo(provenanceInfo);
		
		// constraint 2: there must not exist two or more instances
		vf.reset();

		List<SPARQLVariable> vars = new LinkedList<SPARQLVariable>();
		SPARQLVariable varS1 = vf.freshSVariable();
		SPARQLVariable varS2 = vf.freshSVariable();
		vars.add(varS1);
		vars.add(varS2);

		FOFormula premise = new FOAnd(
			new FOTriple(varS1, RDF_TYPE, clazz),
			new FOTriple(varS2, RDF_TYPE, clazz));

		FOFormula conclusion = new FOEquals(
				new FOIRIOrSPARQLVariable(varS1), 
				new FOIRIOrSPARQLVariable(varS2));
		
		FOFormula max1 = new FOForall(vars,new FOImplies(premise, conclusion));
		max1.setProvenanceInfo(provenanceInfo);
		
		ret.add(min1);
		ret.add(max1);
		
		
		return ret;
	}

	public static List<FOFormula> classKeyConstraint(
			IRI clazz, Key key, ConstraintProvenanceInfo provenanceInfo) {
		
		List<FOFormula> ret = new LinkedList<FOFormula>();
		
		// set up induced totality constraints for the key properties
		for (IRIWithRangeType iwrt : key.getKeyProps()) {
			IRI keyProp = iwrt.getIRI();
			ret.addAll(totalityConstraint(clazz, keyProp, provenanceInfo));

			if (iwrt.getRangeType()!=null)
				ret.addAll(FOConstraintBuilder.rangeTypeConstraint(clazz, iwrt, provenanceInfo)); 
		}
		
		vf.reset();

		// add the key constraint itself enforcing uniqueness
		List<IRIWithRangeType> keyProps = key.getKeyProps();	
		List<SPARQLVariable> sVars = vf.freshSVariables(2);
		List<SPARQLVariable> oVars = vf.freshOVariables(keyProps.size());

		List<SPARQLVariable> forallVars = new LinkedList<SPARQLVariable>();
		forallVars.addAll(sVars);
		forallVars.addAll(oVars);
		
		// build premise
		List<FOFormula> conjunction = new LinkedList<FOFormula>();
		for (SPARQLVariable sVar : sVars) {
			conjunction.add(new FOTriple(sVar, RDF_TYPE, clazz));
			for (int i=0; i<keyProps.size(); i++) {
				conjunction.add(new FOTriple(sVar,keyProps.get(i).getIRI(),oVars.get(i)));
			}
		}
		FOFormula premise = new FOAnd(conjunction);
		
		// build conclusion
		FOFormula conclusion = 
			new FOEquals(
				new FOIRIOrSPARQLVariable(sVars.get(0)),
				new FOIRIOrSPARQLVariable(sVars.get(1)));

		// build the full constraint
		FOFormula uniquenessConstraint =
			new FOForall(forallVars,
				new FOImplies(premise, conclusion));

		// .. and finally add it to the set of constraints
		ret.add(uniquenessConstraint);
	
		// set (and possibly override with identical) provenance information
		for (FOFormula f : ret) {
			f.setProvenanceInfo(provenanceInfo);
		}
		return ret;
	}
	
	/**
	 * Constructs an FOL formula asserting that the variables are
	 * pairwise distinct. Expects at least two parameters.
	 */
	protected static List<FONot> allDist(List<SPARQLVariable> vars) 
	throws IllegalArgumentException {
		if (vars==null || vars.size()<2)
			throw new IllegalArgumentException("Must pass at least two variables, but vars=" + vars);
		
		List<FONot> inequalities = new LinkedList<FONot>();
		for (int i=0;i<vars.size(); i++) {
			for (int j=i+1; j<vars.size(); j++) {
				inequalities.add(new FONot(
					new FOEquals(
							new FOIRIOrSPARQLVariable(vars.get(i)),
							new FOIRIOrSPARQLVariable(vars.get(j)))));
			}
		}
		
		return inequalities;
	}
	
	/**
	 * Constructs an FOL formula asserting that the variables are
	 * not pairwise distinct. Expects at least two parameters.
	 */
	protected static FOFormula someEqual(List<SPARQLVariable> vars) {
		if (vars==null || vars.size()<2)
			throw new IllegalArgumentException("Must pass at least two variables, but vars=" + vars);
		
		List<FOEquals> equalities = new LinkedList<FOEquals>();
		for (int i=0;i<vars.size(); i++) {
			for (int j=i+1; j<vars.size(); j++) {
				equalities.add(new FOEquals(
					new FOIRIOrSPARQLVariable(vars.get(i)),
					new FOIRIOrSPARQLVariable(vars.get(j))));
			}
		}
		
		if (equalities.size()==1)
			return equalities.get(0);
		else {
			List<FOFormula> casted = new LinkedList<FOFormula>();
			casted.addAll(equalities);
			return new FOOr(casted);
		}
	}
	
	protected static List<FOFormula> convertToListAndSetProvenance(
			FOFormula constraint, ConstraintProvenanceInfo provenanceInfo) {
		LinkedList<FOFormula> ret = new LinkedList<FOFormula>();
		if (constraint!=null) {
			if (provenanceInfo!=null)
				constraint.setProvenanceInfo(provenanceInfo);
			ret.add(constraint);
		}
		return ret;
	}
}