/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.model;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import edu.freiburg.dbis.rdd.fol.FOConstraintBuilder;
import edu.freiburg.dbis.rdd.fol.FOConvertable;
import edu.freiburg.dbis.rdd.fol.FOFormula;
import edu.freiburg.dbis.rdd.provenance.ConstraintProvenanceStringBuilder;
import edu.freiburg.dbis.rdd.semantics.Environment;
import edu.freiburg.dbis.rdd.sparql.IRI;

/**
 * Representation of RDD CLASS constraints
 *
 * @author MSchmidt
 */
public class ClassConstraint implements FOConvertable {

	protected IRI clazz;
	protected List<IRI> subClazzes;
	protected boolean isSingleton;
	protected boolean isOWA;
	protected List<Key> keys;
	protected List<PropConstraint> qpcs;

	public ClassConstraint(WA wa, boolean isSingleton, IRI clazz, List<IRI> subClazzes, List<PropConstraint> qpcs, List<Key> keys) {
		isOWA = WA.OWA.equals(wa);
		this.isSingleton = isSingleton;
		this.clazz = clazz;
		this.subClazzes = subClazzes;
		this.qpcs = qpcs;
		this.keys = keys;
	}
	
	/**
	 * Copy constructor for replication. Replaces the class IRI by clazz and
	 * sets the isSingleton flag to false and the isOWA flag to true,
	 * according to the replication semantics in the paper.
	 */
	public ClassConstraint(IRI clazz, ClassConstraint classConstraint) {
		this.isOWA = true;
		this.isSingleton = false;
		this.clazz = clazz;
		this.subClazzes = classConstraint.getSubClazzes();
		this.qpcs = classConstraint.getPropConstraints();
		this.keys = classConstraint.getKeys();
	}
	
	public IRI getClazz() {
		return clazz;
	}



	public List<IRI> getSubClazzes() {
		return subClazzes;
	}



	public boolean isSingleton() {
		return isSingleton;
	}



	public boolean isOWA() {
		return isOWA;
	}



	public List<Key> getKeys() {
		return keys;
	}



	public List<PropConstraint> getPropConstraints() {
		return qpcs;
	}


	/**
	 * @return a list of all properties constrained in the class
	 * section, either as property constraints or within keys
	 */
	public Collection<IRI> mentionedProps() {

		Collection<IRI> ret = new HashSet<IRI>();
		
		for (PropConstraint pc : getPropConstraints()) {
			ret.add(pc.getPropWithRange().getIRI());
		}

		for (Key key : getKeys()) {
			for (IRIWithRangeType iwrt : key.getKeyProps()) {
				ret.add(iwrt.getIRI());
			}
		}
		
		return ret;
	}

	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		if (isOWA)
			buf.append("OWA");
		else buf.append("CWA");
		if (isSingleton)
			buf.append(" SINGLETON");
		buf.append(" CLASS " + clazz.toString());
		if (!subClazzes.isEmpty()) {
			buf.append(" SUBCLASS ");
			int i=0;
			for (IRI subClazz : subClazzes) {
				if (i++>0)
					buf.append(", ");
				buf.append(subClazz);
			}
		}
		buf.append(" {\n");
		for (Key key : keys) {
			buf.append(key);
			buf.append("\n");
		}
		for (PropConstraint qpc : qpcs) {
			buf.append(qpc);
			buf.append("\n");
		}
		buf.append("}");
		return buf.toString();
	}

	@Override
	public Set<FOFormula> toFOL(Environment e) {
		return toFOL(e, new HashSet<IRI>());
	}

	/**
	 * Helper method for the evaluation phase, maintaining a list of handled
	 * classes to break endless recursion in case of cyclic class hierarchies.
	 */
	public Set<FOFormula> toFOL(Environment e, Set<IRI> handled) {

		Set<FOFormula> ret = new HashSet<FOFormula>();
		
		// cycle breaking
		if (handled.contains(clazz))
			return ret; // stop recursion
		handled.add(clazz); // this will be handled in the following
		
		// class OWA constraint
		if (!isOWA) {
			ret.addAll(
				FOConstraintBuilder.propertyCwaConstraint(
					clazz, mentionedProps(),
					ConstraintProvenanceStringBuilder.propertyCWAConstraint(clazz)));
		}
		
		// class singleton constraint
		if (isSingleton())
			ret.addAll(
				FOConstraintBuilder.classSingletonConstraint(
					clazz, ConstraintProvenanceStringBuilder.classSingletonConstraint(clazz)));

		// class key constraints
		for (Key key : getKeys()) {
			ret.addAll(
				FOConstraintBuilder.classKeyConstraint(
					clazz, key, 
					ConstraintProvenanceStringBuilder.classKeyConstraint(clazz, key)));
		}
		
		// class properties (qualified prop constraints)
		for (PropConstraint pc : getPropConstraints()) {
			ret.addAll(pc.toFOL(getClazz(), e));
		}
		
		// recursion step replicate the classes and add them recursively
		for (IRI subClazz : e.C(clazz)) {
			ClassConstraint inheritedCC = new ClassConstraint(subClazz, this);
			ret.addAll(inheritedCC.toFOL(e, handled));
		}
		
		return ret;
	}
}
