/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.model;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import edu.freiburg.dbis.rdd.fol.FOConstraintBuilder;
import edu.freiburg.dbis.rdd.fol.FOConvertable;
import edu.freiburg.dbis.rdd.fol.FOFormula;
import edu.freiburg.dbis.rdd.provenance.ConstraintProvenanceStringBuilder;
import edu.freiburg.dbis.rdd.semantics.Environment;
import edu.freiburg.dbis.rdd.sparql.IRI;

/**
 * Representation of the PROPERTIES constraint section
 * 
 * @author MSchmidt
 */
public class PropConstraintSec implements FOConvertable {
	protected List<PropConstraint> pcs;
	protected boolean isOWA;
	
	public PropConstraintSec(WA worldAssumption, List<PropConstraint> pcs) {
		isOWA = WA.OWA.equals(worldAssumption);	
		this.pcs = pcs;
	}
	
	public List<PropConstraint> getPropConstraints() {
		return pcs;
	}
	
	public boolean isOWA() {
		return isOWA;
	}
	
	/**
	 * @return a list of all properties constrained in the properties section
	 */
	public Collection<IRI> mentionedProps() {

		Set<IRI> props = new HashSet<IRI>();
		for (PropConstraint pc : pcs) {
			props.add(pc.getPropWithRange().getIRI()); // collect property IRIs
		}
		return props;
	}
	
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		if (isOWA)
			buf.append("OWA");
		else
			buf.append("CWA");
		buf.append(" PROPERTIES { \n");
		for (PropConstraint pc : pcs) {
			buf.append("  " + pc.toString() + "\n");
		}
		buf.append("}");
		return buf.toString();
	}

	@Override
	public Set<FOFormula> toFOL(Environment e) {

		Set<FOFormula> ret = new HashSet<FOFormula>();
		
		// STEP 1: set up constraints imposed by individual props
		for (PropConstraint pc : pcs) {
			ret.addAll(pc.toFOL(null,e));
		}
		
		// STEP 2: set up WA constraint if section defined CWA
		if (!isOWA)
			ret.addAll(
				FOConstraintBuilder.propertyCwaConstraint(null, mentionedProps(),
				ConstraintProvenanceStringBuilder.propertyCWAConstraint(null)));
		return ret;
	}
}