/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.semantics;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import edu.freiburg.dbis.rdd.model.ClassConstraint;
import edu.freiburg.dbis.rdd.model.ClassConstraintSec;
import edu.freiburg.dbis.rdd.model.PropConstraint;
import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.model.SubPropertyConstraint;
import edu.freiburg.dbis.rdd.sparql.IRI;

/**
 * Environment providing access to subclass, subproperty
 * and class section property information for a given RDD.
 * 
 * @author MSchmidt
 */
public class Environment {

	protected RDD rdd; 

	/** Subclass relationships */
	protected Map<IRI, Collection<IRI>> C;

	/** Subpropery relationships */
	protected Map<IRI, Collection<IRI>> P;

	/** Mapping from classes to properties defined within */
	protected Map<IRI, Collection<IRI>> A;
	
	
	/**
	 * Constructs and initializes a new environment for the given RDD.
	 */
	public Environment(RDD rdd) {
		this.rdd = rdd;
		init();
	}

	/**
	 * Access to the subclass hierarchy (non-recursive).
	 */
	public Collection<IRI> C(IRI clazz) {
		Collection<IRI> subClazzez = C.get(clazz);
		return subClazzez == null ? new HashSet<IRI>() : subClazzez;
	}

	/**
	 * Access to the subproperty hierarchy (non-recursive).
	 */
	public Collection<IRI> P(IRI property) {
		Collection<IRI> subProperties = P.get(property);
		return subProperties == null ? new HashSet<IRI>() : subProperties;
	}
	
	/**
	 * Access to the class properties (non-recursive).
	 */
	public Collection<IRI> A(IRI clazz) {
		Collection<IRI> clazzProperties = P.get(clazz);
		return clazzProperties == null ? new HashSet<IRI>() : clazzProperties;
	}

	/**
	 * Initializes the environment by making one pass over the RDD
	 * and collecting the relevant information from it.
	 */
	protected void init() {
		C = new HashMap<IRI, Collection<IRI>>();
		P = new HashMap<IRI, Collection<IRI>>();
		A = new HashMap<IRI, Collection<IRI>>();
		
		List<PropConstraint> allPropConstraints = new LinkedList<PropConstraint>();
		ClassConstraintSec ccs = rdd.getClassConstraintSec();
		if (ccs!=null) {
			for (ClassConstraint cc : ccs.getClassConstraints()) {
				
				// set up C relationship
				List<IRI> currentSubClazzez = cc.getSubClazzes();
				IRI clazz = cc.getClazz();
				Collection<IRI> subClazzez = C.get(clazz);
				if (subClazzez==null)
					subClazzez = new HashSet<IRI>();
				subClazzez.addAll(currentSubClazzez);
				C.put(clazz, subClazzez);
				
				// set up A relationship
				Collection<IRI> classProperties = A.get(clazz);
				if (classProperties==null)
					classProperties = new HashSet<IRI>();
				classProperties.addAll(cc.mentionedProps());
				A.put(clazz, classProperties);
				
				allPropConstraints.addAll(cc.getPropConstraints());
			}
		}
		
		// set up P relationship
		if (rdd.getPropConstraintSec()!=null) {
			allPropConstraints.addAll(rdd.getPropConstraintSec().getPropConstraints());
			for (PropConstraint pc : allPropConstraints) {
				if (pc instanceof SubPropertyConstraint) {
					SubPropertyConstraint pcCast = (SubPropertyConstraint)pc;
					IRI constrainedProp = pc.getPropWithRange().getIRI();
					Collection<IRI> subProperties = P.get(constrainedProp);
					if (subProperties==null)
						subProperties = new HashSet<IRI>();
					subProperties.addAll(pcCast.getSubProps());
					P.put(constrainedProp, subProperties);
				}
			}
		}
	}
}
