/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.sparql;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.openrdf.OpenRDFException;
import org.openrdf.model.Value;
import org.openrdf.query.BindingSet;
import org.openrdf.query.BooleanQuery;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.TupleQueryResult;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.rio.RDFFormat;
import org.openrdf.sail.memory.MemoryStore;

/**
 * General RDF data management functionality
 * 
 * @author MSchmidt
 */
public class DataManager {

	private static Logger logger = Logger.getLogger(DataManager.class);
	
	private Repository repo;
	
	/**
	 * Initializes the data manager with the file from
	 * which to load the repository (into main memory)
	 * 
	 * @param rdfFile
	 * @throws RepositoryException 
	 */
	public DataManager(File rdfFile, RDFFormat rdfFormat) throws RepositoryException {
		initRepository(rdfFile, rdfFormat);
	}
	
	protected void initRepository(File rdfFile, RDFFormat rdfFormat) 
	throws RepositoryException {
		
		repo = new SailRepository(new MemoryStore());
		repo.initialize();
		
		try {
			RepositoryConnection con = repo.getConnection();
			try {
				con.add(rdfFile, null, rdfFormat);
			} finally {
				con.close();
			}
		}
		catch (OpenRDFException e) {
			logger.error(e.getMessage());
			throw new RuntimeException(e);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new RuntimeException(e);
		}
	}
	
	public void shutdown() {
		try {
			repo.shutDown();
		} catch (RepositoryException e) {
			logger.error(e.getMessage());
		}
	}
	
	public boolean sparqlAsk(String askQuery) {
		
		Boolean res = false;
		try {
			RepositoryConnection con = repo.getConnection();
			try {
				BooleanQuery booleanQuery = 
					con.prepareBooleanQuery(QueryLanguage.SPARQL, askQuery);
				res = booleanQuery.evaluate();
			} finally {
				con.close();
			}
		} catch (OpenRDFException e) {
			logger.error(e.getMessage());
			throw new RuntimeException(e);
		}
		
		return res;
	}
	
	public List<Map<String,Value>> sparqlSelect(String selectQuery) {

		List<Map<String,Value>> res = new LinkedList<Map<String,Value>>();
		try {
			RepositoryConnection con = repo.getConnection();
			try {
				TupleQuery tupleQuery = 
					con.prepareTupleQuery(QueryLanguage.SPARQL, selectQuery);
		
				TupleQueryResult result = tupleQuery.evaluate();
				try {
					while (result.hasNext()) {
						BindingSet bindingSet = result.next();
						bindingSet.getBindingNames();
						Map<String,Value> row = new HashMap<String,Value>();
						for (String var : bindingSet.getBindingNames()) {
							row.put(var, bindingSet.getValue(var));
						}
						res.add(row);
					}					
				} finally {
					result.close();
				}
			} finally {
				con.close();
			}
		} catch (OpenRDFException e) {
			logger.error(e.getMessage() + ": \n" + selectQuery);
			throw new RuntimeException(e);
		}
		
		return res;
	}
}