/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.sparql;

import org.apache.commons.lang3.tuple.Pair;

import edu.freiburg.dbis.rdd.model.PrefixSec;
import edu.freiburg.dbis.rdd.parser.PrefixManager;

/**
 * Internal representation of IRIs.
 *
 * @author MSchmidt
 */
public class IRI implements SPARQLTriplePatternComponent {
	
	String fullIRI; 
	PrefixSec prefixSec;
	
	/**
	 * Use {@link #fromFullUri(String)} or {@link #fromPrefixedUri(String)}
	 * to guarantee proper initialization.
	 */
	private IRI() {
	}
	
	/**
	 * Constructs a IRI from a prefixed IRI string using the {@link PrefixManager}
	 */
	public static IRI fromPrefixedIri(String prefixedIri) {
		IRI iri = new IRI();
		
		PrefixSec prefixSec = PrefixManager.getInstance().getPrefixSec();
		iri.prefixSec = prefixSec;

		Pair<String,String> p = splitPrefixedIri(prefixedIri);
		String resolvedPrefix = prefixSec.resolvePrefix(p.getLeft());
		
		if (resolvedPrefix==null)
			throw new IllegalArgumentException("Prefix '" + p.getLeft() + "' undefined.");
		
		iri.fullIRI = resolvedPrefix + p.getRight();
		return iri;
	}
	

	/**
	 * Constructs a IRI from a SPARQL IRI such as <http://www.example.com/MyIRI>
	 */
	public static IRI fromSparqlIri(String fullIri) {
		IRI iri = new IRI();
		iri.fullIRI = fullIri.substring(1,fullIri.length()-1);
		iri.prefixSec = PrefixManager.getInstance().getPrefixSec();
		
		return iri;
	}

	/**
	 * Constructs a IRI from a prefix and a suffix.
	 */
	public IRI(String prefix, String suffix) {
		this.fullIRI = prefix + ":" + suffix;
	}
	
	public String getFullIRI() {
		return fullIRI; 
	}
	
	@Override
	public String toSPARQL() {
		return toString();
	}
	
	@Override
	public String toString() {
		// note: this is used for output, do NOT change
		return "<" + fullIRI + ">";
	}

	/**
	 * Splits a string on its first ":" occurrence.
	 */
	public static Pair<String,String> splitPrefixedIri(String prefixedIri) {

		int indexOfColon = prefixedIri.indexOf(":");
		if (indexOfColon<0)
			throw new IllegalArgumentException("Prefixed IRI " + prefixedIri + " contains no colon.");
		
		return Pair.of(prefixedIri.substring(0, indexOfColon),
						prefixedIri.substring(indexOfColon+1));
	}
	
	/**
	 * Define equality over the fullUri, which defines
	 * the URI uniquely.
	 */
	@Override
	public int hashCode() {
		return fullIRI.hashCode();
	}
	
	/**
	 * Define equality over the fullUri, which defines
	 * the URI uniquely.
	 */
	@Override
	public boolean equals(Object obj) {
        if (obj == null)
            return false;
        if (obj == this)
            return true;
        if (!(obj instanceof IRI))
            return false;

        return this.hashCode()==obj.hashCode();
	}
}
