/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.sparql;

import java.util.ArrayList;
import java.util.List;

/**
 * Factory singleton class to create SPARQL variables
 * 
 * @author MSchmidt
 */
public class SPARQLVariableFactory {

	private static SPARQLVariableFactory instance = null;

	int subjectCtr;
	int predicateCtr;
	int objectCtr;
	int dummyCtr;
	
	/**
	 * Retrieves the one and only instance of the class
	 */
	public static synchronized SPARQLVariableFactory getInstance() {
		if (instance==null)
			instance = new SPARQLVariableFactory();
		return instance;
	}

	/**
	 * Private singleton constructor
	 */
	private SPARQLVariableFactory() {
		reset(); // = initialize
	}
	
	/**
	 * @return one fresh subject position variable
	 */
	public synchronized SPARQLVariable freshSVariable() {
		return new SPARQLVariable("s" + subjectCtr++);
	}

	/**
	 * @return n fresh subject position variables
	 */
	public synchronized List<SPARQLVariable> freshSVariables(int n) {
		List<SPARQLVariable> vars = new ArrayList<SPARQLVariable>(n);
		for (int i=0; i<n; i++) {
			vars.add(freshSVariable());
		}
		return vars;
	}
	
	/**
	 * @return one fresh predicate position variable
	 */
	public synchronized SPARQLVariable freshPVariable() {
		return new SPARQLVariable("p" + predicateCtr++);
	}
	
	/**
	 * @return n fresh predicate position variables
	 */
	public synchronized List<SPARQLVariable> freshPVariables(int n) {
		List<SPARQLVariable> vars = new ArrayList<SPARQLVariable>(n);
		for (int i=0; i<n; i++) {
			vars.add(freshPVariable());
		}
		return vars;
	}

	/**
	 * @return one fresh object position variable
	 */
	public synchronized SPARQLVariable freshOVariable() {
		return new SPARQLVariable("o" + objectCtr++);
	}
	
	/**
	 * @return n fresh subject position variables
	 */
	public synchronized List<SPARQLVariable> freshOVariables(int n) {
		List<SPARQLVariable> vars = new ArrayList<SPARQLVariable>(n);
		for (int i=0; i<n; i++) {
			vars.add(freshOVariable());
		}
		return vars;
	}
	
	/**
	 * Creates a fresh dummy variable
	 */
	public synchronized SPARQLVariable freshDummyVariable() {
		return new SPARQLVariable("v" + dummyCtr++);
	}
	
	/**
	 * Resets the counter. Should be called before creating a new SPARQL query.
	 */
	public void reset() {
		subjectCtr = 1;
		predicateCtr = 1;
		objectCtr = 1;
		dummyCtr = 1;
	}
}