/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.parser;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.List;

import org.junit.Test;

import edu.freiburg.dbis.rdd.model.ClassConstraint;
import edu.freiburg.dbis.rdd.model.IRIWithRangeType;
import edu.freiburg.dbis.rdd.model.Key;
import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.model.RangeTypeBNode;
import edu.freiburg.dbis.rdd.model.RangeTypeIRI;
import edu.freiburg.dbis.rdd.model.RangeTypeLiteral;
import edu.freiburg.dbis.rdd.model.RangeTypeResource;

/**
 * Test parsing of class KEYs, make sure class keys do
 * no occur in PROPERTIES section.
 * 
 * @author MSchmidt
 */
public class ClassKeyTest {

	private static String TEST_BASE_DIR = "examples/test/parse/";
	
	List<Class<?>> constraintTypes;
	
	@Test
	public void testClassKey()
	throws FileNotFoundException, ParseException {
		String rddFile = TEST_BASE_DIR + "ClassKey.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    System.out.println(rdd);
	    System.out.println("\n\n");

	    List<ClassConstraint> ccs = rdd.getClassConstraintSec().getClassConstraints();
	    
	    assertEquals(2,ccs.size());

	    // Class 1 contains:
	    // KEY <http://www.example.com/p1> : IRI; 	  
	    List<Key> keysC1 = ccs.get(0).getKeys();
	    assertEquals(1,keysC1.size());

	    Key keyC1 = keysC1.get(0);
	    List<IRIWithRangeType> keyProps1 = keyC1.getKeyProps();

	    assertEquals(1, keyProps1.size());
	    assertEquals("<http://www.example.com/p1>", keyProps1.get(0).getIRI().toString());
	    assertTrue(keyProps1.get(0).getRangeType() instanceof RangeTypeIRI);
	    
	    
	    // Class 2 contains:
	    //     KEY <http://www.example.com/p1> : IRI, 
	    //         <http://www.example.com/p2> : BNODE, 
	    //         <http://www.example.com/p3> : LITERAL(<http://my.datatype/int>), 
	    //         <http://www.example.com/p4> : RESOURCE, 
	    //         <http://www.example.com/p5>; 
	    List<Key> keysC2 = ccs.get(1).getKeys();
	    assertEquals(1,keysC2.size());
	    
	    Key keyC2 = keysC2.get(0);
	    List<IRIWithRangeType> keyProps2 = keyC2.getKeyProps();
	    
	    assertEquals(5, keyProps2.size());
	    assertEquals("<http://www.example.com/p1>", keyProps2.get(0).getIRI().toString());
	    assertEquals("<http://www.example.com/p2>", keyProps2.get(1).getIRI().toString());
	    assertEquals("<http://www.example.com/p3>", keyProps2.get(2).getIRI().toString());
	    assertEquals("<http://www.example.com/p4>", keyProps2.get(3).getIRI().toString());
	    assertEquals("<http://www.example.com/p5>", keyProps2.get(4).getIRI().toString());
	    assertTrue(keyProps2.get(0).getRangeType() instanceof RangeTypeIRI);
	    assertTrue(keyProps2.get(1).getRangeType() instanceof RangeTypeBNode);
	    assertTrue(keyProps2.get(2).getRangeType() instanceof RangeTypeLiteral);
	    assertTrue(keyProps2.get(3).getRangeType() instanceof RangeTypeResource);
	    assertTrue(keyProps2.get(4).getRangeType()==null);
	}
	
	
	@Test
	public void testKeyInPropSecFailing()
	throws FileNotFoundException, ParseException {
		String rddFile = TEST_BASE_DIR + "KeyInPropSecFailing.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    
	    try {
	    	parser.rdd();	
	    } catch (ParseException e) {
	    	return; // success
	    }
	    
	    throw new RuntimeException("Parsing succeeded, but should "
	    		+ "fail due to KEY in property section!");
	}
}
