/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.parser;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.List;

import org.junit.Test;

import edu.freiburg.dbis.rdd.model.PrefixDecl;
import edu.freiburg.dbis.rdd.model.RDD;

/**
 * Tests asserting the equivalence of the use of prefixed
 * and full IRIs.
 * 
 * @author MSchmidt
 */
public class PrefixSectionTest {

	private static String TEST_BASE_DIR = "examples/test/parse/";

	@Test
	public void testPrefixSectionParsing()
	throws FileNotFoundException, ParseException {
		
		// STEP 1: first parsing round, and check that prefixes
		// are parsed correctly and prefix manager is properly
		// set up after parsing
		String rddFile = TEST_BASE_DIR + "PrefixSec123.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	
	    
	    List<PrefixDecl> prefixes123 = rdd.getPrefixSec().getPrefixes();	    
	    List<PrefixDecl> prefixesManager = 
	    		PrefixManager.getInstance().getPrefixSec().getPrefixes();

	    assertPrefixes123(prefixes123);
	    assertPrefixes123(prefixesManager);	    
	    
		// STEP 1: parse another file, which should work properly,
	    // not modify the prefix section of the first one, but the
	    // prefix managers section
		rddFile = TEST_BASE_DIR + "PrefixSec34.rdd";
		fis = new FileInputStream(new File(rddFile));

	    parser = new RDDParser(fis);
	    rdd = parser.rdd();	
	    
	    List<PrefixDecl> prefixes34 = rdd.getPrefixSec().getPrefixes();	    
	    prefixesManager = 
	    		PrefixManager.getInstance().getPrefixSec().getPrefixes();

	    assertPrefixes34(prefixes34);
	    
	    assertPrefixes123(prefixes123); // unchanged
	    assertPrefixes34(prefixesManager); // changed
	}

	/**
	 * Checks that the prefixes are defined according to the
	 * example file as follows:
	 *
	 * <code>
	 * PREFIX ex1: <http://www.example1.com/>
     * PREFIX ex3: <http://www.example2.com#>
     * PREFIX ex2: <http://www.example3.com/>
     * </code>
     *
	 * @param prefixes
	 */
	private void assertPrefixes123(List<PrefixDecl> prefixes) {
		assertEquals(3,prefixes.size());
		boolean match1 = false;
		boolean match2 = false;
		boolean match3 = false;
		for (PrefixDecl prefix : prefixes) {
			match1 |= prefix.getPrefix().equals("ex1") && prefix.getIri().equals("http://www.example1.com/");
			match2 |= prefix.getPrefix().equals("ex2") && prefix.getIri().equals("http://www.example2.com#");
			match3 |= prefix.getPrefix().equals("ex3") && prefix.getIri().equals("http://www.example3.com/");
		}
		assertTrue(match1 && match2 && match3);
	}
	
	private void assertPrefixes34(List<PrefixDecl> prefixes) {
		assertEquals(2,prefixes.size());
		boolean match1 = false;
		boolean match2 = false;
		for (PrefixDecl prefix : prefixes) {
			match1 |= prefix.getPrefix().equals("ex3") && prefix.getIri().equals("http://www.example3.com.new/");
			match2 |= prefix.getPrefix().equals("ex4") && prefix.getIri().equals("http://www.example4.com.new/");
		}
		assertTrue(match1 && match2);
	}

}
