/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.parser;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.List;

import org.apache.commons.lang3.tuple.Pair;
import org.junit.Test;

import edu.freiburg.dbis.rdd.model.ClassConstraint;
import edu.freiburg.dbis.rdd.model.MinConstraint;
import edu.freiburg.dbis.rdd.model.PathConstraint;
import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.sparql.IRI;

/**
 * Tests covering prefixed IRI splitting and processing,
 * including failures due to the use of undefined prefixes.
 * 
 * @author MSchmidt
 */
public class PrefixedIRITest {

	private static String TEST_BASE_DIR = "examples/test/parse/";

	@Test
	public void testSplitFunction() {
	    Pair<String,String> p1 = IRI.splitPrefixedIri("abcd:efg");
	    assertEquals("abcd",p1.getLeft());
	    assertEquals("efg",p1.getRight());
	    
	    Pair<String,String> p2 = IRI.splitPrefixedIri("a:b");
	    assertEquals("a",p2.getLeft());
	    assertEquals("b",p2.getRight());

	    Pair<String,String> p3 = IRI.splitPrefixedIri(":abc");
	    assertEquals("",p3.getLeft());
	    assertEquals("abc",p3.getRight());

	    Pair<String,String> p4 = IRI.splitPrefixedIri("abc:");
	    assertEquals("abc",p4.getLeft());
	    assertEquals("",p4.getRight());
	}
	
	@Test
	public void testPrefixedIRIsParsing()
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "PrefixedIRIs.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    List<ClassConstraint> ccs = rdd.getClassConstraintSec().getClassConstraints();
	    assertEquals(1,ccs.size());

	    ClassConstraint cc = ccs.get(0);
	    assertEquals("<http://www.test.com/a>", cc.getClazz().toString());

    	assertEquals(2,cc.getPropConstraints().size());
    	PathConstraint cPath = (PathConstraint)cc.getPropConstraints().get(0);
    	MinConstraint cMin = (MinConstraint)cc.getPropConstraints().get(1);
    	
    	assertEquals(2,cPath.getPath().size());
    	assertEquals("<http://www.test2.com/>",cPath.getPath().get(0).toString());
    	assertEquals("<http://www.test.com/c>",cPath.getPath().get(1).toString());
    	assertEquals("<http://www.test.com/>",cPath.getPropWithRange().getIRI().toString());

    	assertEquals("<http://www.default.com/x>",cMin.getPropWithRange().getIRI().toString());
	}
	
	@Test
	public void testPrefixedIRIsFailingParsing()
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "PrefixedIRIsFailing.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    
	    boolean failed = false;
	    try {
	    	parser.rdd();
	    } catch (IllegalArgumentException e) {
	    	// there is an illegal prefix used in the file
	    	failed = true;
	    }
	    
	    assertTrue(failed);
	}
}
