/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.semantics;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Set;

import org.junit.Test;

import edu.freiburg.dbis.rdd.fol.FOFormula;
import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.parser.ParseException;
import edu.freiburg.dbis.rdd.parser.RDDParser;

/**
 * Test the semantics translation for the individual constraint types,
 * such as Domain, Range, Min, Max, etc. in the context of property and
 * class sections.
 * 
 * @author MSchmidt
 */
public class ConstraintTypeTest {

	protected static String TEST_BASE_DIR = "examples/test/semantics/";

	@Test
	public void testDomainConstraint() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "DomainConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASS <http://www.example.com/C1> {
	     *   DOMAIN(<http://www.example.com/C2>) <http://www.example.com/p1>;
	     * }
	     */
	    String qualifiedDomainConstraint = 
	    	Util.serializedDomainConstraint(Util.CLASS1, Util.PROP1, Util.CLASS2);

	    /**
	     * OWA PROPERTIES { 
	     *   DOMAIN(<http://www.example.com/C3>) <http://www.example.com/p2>;
	     * }
	     */
	    String unqualifiedDomainConstraint = 
		    Util.serializedDomainConstraint(null, Util.PROP2, Util.CLASS3);

	    Util.assertSemantics(formulas, unqualifiedDomainConstraint, qualifiedDomainConstraint);
	}
	
	@Test
	public void testRangeConstraint() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "RangeConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASS <http://www.example.com/C1> {
	     *   RANGE(<http://www.example.com/C2>) <http://www.example.com/p1>;
	     * }
	     */
	    String qualifiedRangeConstraint = 
	    	Util.serializedRangeConstraint(Util.CLASS1, Util.PROP1, Util.CLASS2);

	    /**
	     * OWA PROPERTIES { 
	     *   RANGE(<http://www.example.com/C3>) <http://www.example.com/p2>;
	     * }
	     */
	    String unqualifiedRangeConstraint = 
		    Util.serializedRangeConstraint(null, Util.PROP2, Util.CLASS3);

	    Util.assertSemantics(formulas, qualifiedRangeConstraint, unqualifiedRangeConstraint);
	}

	@Test
	public void testMinConstraint()
	throws FileNotFoundException, ParseException {

		String rddFile = TEST_BASE_DIR + "MinConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

		RDDParser parser = new RDDParser(fis);
		RDD rdd = parser.rdd();

		Set<FOFormula> formulas = rdd.toFOL();

		/**
		 * OWA CLASS <http://www.example.com/C1> {
		 *   MIN(1) <http://www.example.com/p1>;
		 *   MIN(3) <http://www.example.com/p2>;
		 * }
		 */
		String qualifiedMin1Constraint = 
		    Util.serializedMin1Constraint(Util.CLASS1, Util.PROP1);
		String qualifiedMin3Constraint = 
			Util.serializedMin3Constraint(Util.CLASS1, Util.PROP2);

		/**
		 * OWA PROPERTIES { 
		 *   MIN(1) <http://www.example.com/p3>;
		 *   MIN(3) <http://www.example.com/p4>;
		 * }
		 */
		String unqualifiedMin1Constraint = 
			Util.serializedMin1Constraint(null, Util.PROP3);
		String unqualifiedMin3Constraint = 
			Util.serializedMin3Constraint(null, Util.PROP4);


		Util.assertSemantics(formulas,
				qualifiedMin1Constraint, qualifiedMin3Constraint,
				unqualifiedMin1Constraint, unqualifiedMin3Constraint);
	}

	
	@Test
	public void testMaxConstraint()
	throws FileNotFoundException, ParseException {

		String rddFile = TEST_BASE_DIR + "MaxConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

		RDDParser parser = new RDDParser(fis);
		RDD rdd = parser.rdd();

		Set<FOFormula> formulas = rdd.toFOL();

		/**
		 * OWA CLASS <http://www.example.com/C1> {
		 *   MAX(1) <http://www.example.com/p1>;
		 *   MAX(3) <http://www.example.com/p2>;
		 * }
		 */
		String qualifiedMax1Constraint = 
			Util.serializedMax1Constraint(Util.CLASS1, Util.PROP1);
		String qualifiedMax3Constraint =
			Util.serializedMax3Constraint(Util.CLASS1, Util.PROP2);
		
		/**
		 * OWA PROPERTIES { 
		 *   MAX(1) <http://www.example.com/p3>;
		 *   MAX(3) <http://www.example.com/p4>;
		 * }
		 */
		String unqualifiedMax1Constraint =
			Util.serializedMax1Constraint(null, Util.PROP3);
		String unqualifiedMax3Constraint = 
			Util.serializedMax3Constraint(null, Util.PROP4);

		Util.assertSemantics(formulas,
				qualifiedMax1Constraint, qualifiedMax3Constraint,
				unqualifiedMax1Constraint, unqualifiedMax3Constraint);
	}
	
	@Test
	public void testTotalityConstraint() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "TotalityConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASS <http://www.example.com/C1> {
	     *   TOTAL <http://www.example.com/p1>;
	     * }
	     */
	    String qualifiedTotalityMinConstraint = 
			Util.serializedMin1Constraint(Util.CLASS1, Util.PROP1);
	    String qualifiedTotalityMaxConstraint = 
			Util.serializedMax1Constraint(Util.CLASS1, Util.PROP1);
	    
	    /**
	     * OWA PROPERTIES { 
	     *   TOTAL <http://www.example.com/p2>;
	     * }
	     */
	    String unqualifiedTotalityMinConstraint =
			Util.serializedMin1Constraint(null, Util.PROP2);
	    String unqualifiedTotalityMaxConstraint = 
			Util.serializedMax1Constraint(null, Util.PROP2);

	    Util.assertSemantics(formulas, 
	    	qualifiedTotalityMinConstraint, qualifiedTotalityMaxConstraint,
	    	unqualifiedTotalityMinConstraint, unqualifiedTotalityMaxConstraint);
	}
	
	@Test
	public void testPartialityConstraint() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "PartialityConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASS <http://www.example.com/C1> {
	     *   PARTIAL <http://www.example.com/p1>;
	     * }
	     */
	    String qualifiedPartialityConstraint = 
			Util.serializedMax1Constraint(Util.CLASS1, Util.PROP1);

	    /**
	     * OWA PROPERTIES { 
	     *   PARTIAL <http://www.example.com/p2>;
	     * }
	     */
	    String unqualifiedPartialityConstraint =
			Util.serializedMax1Constraint(null, Util.PROP2);

	    Util.assertSemantics(formulas, qualifiedPartialityConstraint, unqualifiedPartialityConstraint);
	}
	
	@Test
	public void testPathConstraint() throws FileNotFoundException,
			ParseException {

		String rddFile = TEST_BASE_DIR + "PathConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

		RDDParser parser = new RDDParser(fis);
		RDD rdd = parser.rdd();

		Set<FOFormula> formulas = rdd.toFOL();

		/**
		 * OWA CLASS <http://www.example.com/C1> {
		 *   PATH(<http://www.example.com/p2>) <http://www.example.com/p1>;
		 *   PATH(<http://www.example.com/p4>/<http://www.example.com/p5>/<http://www.example.com/p6>) <http://www.example.com/p3>;
		 * }
		 */
		String qualifiedPathConstraint1 = 
			Util.serializedPath1Constraint(Util.CLASS1, Util.PROP1, Util.PROP2);
		String qualifiedPathConstraint2 =
			Util.serializedPath3Constraint(Util.CLASS1, Util.PROP3, Util.PROP4, Util.PROP5, Util.PROP6);

		/**
		 * OWA PROPERTIES { 
		 *   PATH(<http://www.example.com/p8>) <http://www.example.com/p7>;
		 *   PATH(<http://www.example.com/p10>/<http://www.example.com/p11>/<http://www.example.com/p12>) <http://www.example.com/p9>;
		 * }
		 */
		String unqualifiedPathConstraint1 = 
			Util.serializedPath1Constraint(null, Util.PROP7, Util.PROP8);
		String unqualifiedPathConstraint2 = 
				Util.serializedPath3Constraint(null, Util.PROP9, Util.PROP10, Util.PROP11, Util.PROP12);

		Util.assertSemantics(formulas, 
				qualifiedPathConstraint1, qualifiedPathConstraint2,
				unqualifiedPathConstraint1, unqualifiedPathConstraint2);
	}
	
	@Test
	public void testSubPropertyConstraintWithoutInheritance()
	throws FileNotFoundException, ParseException {

		String rddFile = TEST_BASE_DIR + "SubPropertyConstraint.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

		RDDParser parser = new RDDParser(fis);
		RDD rdd = parser.rdd();

		Set<FOFormula> formulas = rdd.toFOL();

		/**
		 * OWA CLASS <http://www.example.com/C1> {
		 *   SUBPROPERTY(<http://www.example.com/p2>) <http://www.example.com/p1>;
		 *   SUBPROPERTY(<http://www.example.com/p4>,<http://www.example.com/p5>,<http://www.example.com/p6>) <http://www.example.com/p3>;
		 * }
		 */
		String qualifiedSubPropertyConstraint1 =
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP1, Util.PROP2);
		String qualifiedSubPropertyConstraint2a =
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP3, Util.PROP4);
		String qualifiedSubPropertyConstraint2b =
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP3, Util.PROP5);
		String qualifiedSubPropertyConstraint2c =
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP3, Util.PROP6);

		/**
		 * OWA PROPERTIES { 
		 *   SUBPROPERTY(<http://www.example.com/p8>) <http://www.example.com/p7>;
		 *   SUBPROPERTY(<http://www.example.com/p10>,<http://www.example.com/p11>,<http://www.example.com/p12>) <http://www.example.com/p9>;
		 * }
		 */
		String unqualifiedSubPropertyConstraint1 =
			Util.serializedSubPropertyConstraint(null, Util.PROP7, Util.PROP8);
		String unqualifiedSubPropertyConstraint2a =
			Util.serializedSubPropertyConstraint(null, Util.PROP9, Util.PROP10);
		String unqualifiedSubPropertyConstraint2b =
			Util.serializedSubPropertyConstraint(null, Util.PROP9, Util.PROP11);
		String unqualifiedSubPropertyConstraint2c =
			Util.serializedSubPropertyConstraint(null, Util.PROP9, Util.PROP12);

		Util.assertSemantics(formulas, 
				qualifiedSubPropertyConstraint1, qualifiedSubPropertyConstraint2a,
				qualifiedSubPropertyConstraint2b, qualifiedSubPropertyConstraint2c,
				unqualifiedSubPropertyConstraint1, unqualifiedSubPropertyConstraint2a,
				unqualifiedSubPropertyConstraint2b, unqualifiedSubPropertyConstraint2c);
	}

	@Test
	public void testSubPropertyConstraintWithInheritance() 
	throws FileNotFoundException, ParseException {

		String rddFile = TEST_BASE_DIR + "SubPropertyConstraintWithInheritance.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

		RDDParser parser = new RDDParser(fis);
		RDD rdd = parser.rdd();

		Set<FOFormula> formulas = rdd.toFOL();

		/*
		 * The RDD looks as follows:
		 * OWA CLASSES {
		 *   OWA CLASS <http://www.example.com/C1> {
		 *     SUBPROPERTY(<http://www.example.com/p2>,<http://www.example.com/p3>) <http://www.example.com/p1>;
		 *     SUBPROPERTY(<http://www.example.com/p4>,<http://www.example.com/p5>) <http://www.example.com/p2>;
		 *   }
		 * }
		 * 
		 * OWA PROPERTIES { 
		 *   SUBPROPERTY(<http://www.example.com/p6>) <http://www.example.com/p5>;    
		 * }
		 * 
		 * The direct subproperty relationships are as follows:
		 * - p1 > p2, p3
		 * - p2 > p4, p5
		 * - p5 > p6 (unqualified)
		 * 
		 * The implied subproperty relationships are as follows:
		 * - p1 > p4, p5, p6
		 * - p2 > p6
		*/
		String p1p2Direct = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP1, Util.PROP2);
		String p1p3Direct = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP1, Util.PROP3);
		String p2p4Direct = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP2, Util.PROP4);
		String p2p5Direct = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP2, Util.PROP5);
		String p5p6Direct = 
			Util.serializedSubPropertyConstraint(null, Util.PROP5, Util.PROP6);
		String p1p4Implied = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP1, Util.PROP4);
		String p1p5Implied = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP1, Util.PROP5);
		String p1p6Implied = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP1, Util.PROP6);
		String p2p6Implied = 
			Util.serializedSubPropertyConstraint(Util.CLASS1, Util.PROP2, Util.PROP6);

		Util.assertSemantics(formulas, 
				p1p2Direct, p1p3Direct, p2p4Direct, p2p5Direct, p5p6Direct,
				p1p4Implied, p1p5Implied, p1p6Implied, p2p6Implied);
	}
	
	@Test
	public void testSubPropertyConstraintWithCyclicInheritance() 
	throws FileNotFoundException, ParseException {

		String rddFile = TEST_BASE_DIR + "SubPropertyConstraintWithCyclicInheritance.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

		RDDParser parser = new RDDParser(fis);
		RDD rdd = parser.rdd();

		Set<FOFormula> formulas = rdd.toFOL();

		/*
		 * OWA PROPERTIES { 
		 *   SUBPROPERTY(<http://www.example.com/p2>,<http://www.example.com/p3>) <http://www.example.com/p1>;
		 *   SUBPROPERTY(<http://www.example.com/p4>,<http://www.example.com/p5>) <http://www.example.com/p2>;
		 *   SUBPROPERTY(<http://www.example.com/p6>) <http://www.example.com/p5>;
		 *   SUBPROPERTY(<http://www.example.com/p1>,<http://www.example.com/p7>) <http://www.example.com/p6>;
		 * }
		 *   
		 * This means we have the following (cyclic) hierarchy
		 * ### Direct relationships:
		 * - p1 > p2, p1 > p3
		 * - p2 > p4, p2 > p5
		 * - p5 > p6
		 * - p6 > p7, p6 > p1
		 * ### Implied relationships (by inheritance):
		 * - p1, p2, p5, p6 are all > p1, p2, p3, p4, p5, p6, p7
		 */
		String p1p2 = 
			Util.serializedSubPropertyConstraint(null, Util.PROP1, Util.PROP2);
		String p1p3 = 
			Util.serializedSubPropertyConstraint(null, Util.PROP1, Util.PROP3);
		String p1p4 =
			Util.serializedSubPropertyConstraint(null, Util.PROP1, Util.PROP4);
		String p1p5 =
			Util.serializedSubPropertyConstraint(null, Util.PROP1, Util.PROP5);
		String p1p6 =
			Util.serializedSubPropertyConstraint(null, Util.PROP1, Util.PROP6);
		String p1p7 =
			Util.serializedSubPropertyConstraint(null, Util.PROP1, Util.PROP7);
		String p2p1 =
			Util.serializedSubPropertyConstraint(null, Util.PROP2, Util.PROP1);
		String p2p3 =
			Util.serializedSubPropertyConstraint(null, Util.PROP2, Util.PROP3);
		String p2p4 =
			Util.serializedSubPropertyConstraint(null, Util.PROP2, Util.PROP4);
		String p2p5 =
			Util.serializedSubPropertyConstraint(null, Util.PROP2, Util.PROP5);
		String p2p6 =
			Util.serializedSubPropertyConstraint(null, Util.PROP2, Util.PROP6);
		String p2p7 =
			Util.serializedSubPropertyConstraint(null, Util.PROP2, Util.PROP7);
		String p5p1 =
			Util.serializedSubPropertyConstraint(null, Util.PROP5, Util.PROP1);
		String p5p2 =
			Util.serializedSubPropertyConstraint(null, Util.PROP5, Util.PROP2);
		String p5p3 =
			Util.serializedSubPropertyConstraint(null, Util.PROP5, Util.PROP3);
		String p5p4 =
			Util.serializedSubPropertyConstraint(null, Util.PROP5, Util.PROP4);
		String p5p6 =
			Util.serializedSubPropertyConstraint(null, Util.PROP5, Util.PROP6);
		String p5p7 =
			Util.serializedSubPropertyConstraint(null, Util.PROP5, Util.PROP7);
		String p6p1 =
			Util.serializedSubPropertyConstraint(null, Util.PROP6, Util.PROP1);
		String p6p2 =
			Util.serializedSubPropertyConstraint(null, Util.PROP6, Util.PROP2);
		String p6p3 = 
			Util.serializedSubPropertyConstraint(null, Util.PROP6, Util.PROP3);
		String p6p4 = 
			Util.serializedSubPropertyConstraint(null, Util.PROP6, Util.PROP4);
		String p6p5 = 
			Util.serializedSubPropertyConstraint(null, Util.PROP6, Util.PROP5);
		String p4p7 = 
			Util.serializedSubPropertyConstraint(null, Util.PROP6, Util.PROP7);
		
		Util.assertSemantics(formulas, 
				p1p2, p1p3, p1p4, p1p5, p1p6, p1p7,
				p2p1, p2p3, p2p4, p2p5, p2p6, p2p7,
				p5p1, p5p2, p5p3, p5p4, p5p6, p5p7,
				p6p1, p6p2, p6p3, p6p4, p6p5, p4p7);
	}
	
	@Test
	public void testSubPropertyConstraintSelfCycle() 
	throws FileNotFoundException, ParseException {

		String rddFile = TEST_BASE_DIR + "SubPropertyConstraintSelfCycle.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

		RDDParser parser = new RDDParser(fis);
		RDD rdd = parser.rdd();

		Set<FOFormula> formulas = rdd.toFOL();
		
		/**
		 * OWA PROPERTIES {
		 *   SUBPROPERTY(<http://www.example.com/p1>, <http://www.example.com/p2>) <http://www.example.com/p1>;
		 * }
		 */
		String subPropertyConstraint =
			Util.serializedSubPropertyConstraint(null, Util.PROP1, Util.PROP2);
		
		Util.assertSemantics(formulas, subPropertyConstraint);
	}
}