/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.semantics;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Set;

import org.junit.Test;

import edu.freiburg.dbis.rdd.fol.FOFormula;
import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.parser.ParseException;
import edu.freiburg.dbis.rdd.parser.RDDParser;

/**
 * Test the implementation of the decomposition algorithm, by means
 * of a test making sure that in the presence of an RDD providing all
 * possible features, all the expected constraints types are present.
 * This is a complex test checking a combination of all possible features.
 * 
 * @author MSchmidt
 */
public class DecompositionTest {

	protected static String TEST_BASE_DIR = "examples/test/semantics/";

	@Test
	public void testDecomposition() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "MixedFeatures.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /*
	     * PREFIX example: <http://www.example.com/>
	     * PREFIX : <http://www.example.com/>
	     * 
	     * CWA CLASSES {
	     *   OWA CLASS :C1 { 
	     *     KEY <http://www.example.com/p1> : IRI;
	     *     KEY <http://www.example.com/p2> : IRI, <http://www.example.com/p3>, :p4 : RESOURCE;
	     *     example:p1 : LITERAL(<http://my.datatypes#integer>);
	     *     DOMAIN(:C4) :p2 : LITERAL;
	     *     RANGE(<http://www.example.com/C5>) <http://www.example.com/p3>;
	     *   }
	     *   
	     *   CWA SINGLETON CLASS <http://www.example.com/C2> { 
	     *     MIN(1), MAX(3) example:p4 : IRI;
	     *     PARTIAL example:p5 : BNODE;
	     *     TOTAL <http://www.example.com/p6>;
	     *   }
	     *   
	     *   OWA CLASS :C3 SUBCLASS :C4 { 
	     *     PATH(<http://www.example.com/p10>) <http://www.example.com/p7> : RESOURCE;
	     *     SUBPROPERTY(<http://www.example.com/p11>,example:p12,:p13) example:p8;
	     *   }
	     * }
	     */
	    /////////////////////// CLASSES
	    String classes = Util.serializedClassesCWA3Constraint(Util.CLASS1, Util.CLASS3, Util.CLASS2);
	    
	    /////////////////////// CLASS C1
	    ///////////// CLASS C1 KEY 
	    String classC1key1Min = Util.serializedMin1Constraint(Util.CLASS1, Util.PROP1);
	    String classC1key1Max = Util.serializedMax1Constraint(Util.CLASS1, Util.PROP1);
	    String classC1key1Key = Util.serializedKey1Constraint(Util.CLASS1, Util.PROP1);
	    String classC1key1RangeType = Util.serializedRangeTypeConstraint(Util.CLASS1, Util.PROP1, "IRI", null);

	    String classC1key2aMin = Util.serializedMin1Constraint(Util.CLASS1, Util.PROP2);
	    String classC1key2bMin = Util.serializedMin1Constraint(Util.CLASS1, Util.PROP3);
	    String classC1key2cMin = Util.serializedMin1Constraint(Util.CLASS1, Util.PROP4);
	    String classC1key2aMax = Util.serializedMax1Constraint(Util.CLASS1, Util.PROP2);
	    String classC1key2bMax = Util.serializedMax1Constraint(Util.CLASS1, Util.PROP3);
	    String classC1key2cMax = Util.serializedMax1Constraint(Util.CLASS1, Util.PROP4);
	    String classC1key2Key = Util.serializedKey3Constraint(Util.CLASS1, Util.PROP2, Util.PROP3, Util.PROP4);
	    String classC1key2RangeType1 = Util.serializedRangeTypeConstraint(Util.CLASS1, Util.PROP2, "IRI", null);
	    String classC1key2RangeType2 = Util.serializedRangeTypeConstraint(Util.CLASS1, Util.PROP4, "RESOURCE", null);

	    ///////////// CLASS C1 PROPS
	    String classC1PropP1RangeType = Util.serializedRangeTypeConstraint(Util.CLASS1, Util.PROP1, "LITERAL", Util.INTEGER);
	    String classC1PropP2RangeType = Util.serializedRangeTypeConstraint(Util.CLASS1, Util.PROP2, "LITERAL", null);
	    String classC1PropP2Domain= Util.serializedDomainConstraint(Util.CLASS1, Util.PROP2, Util.CLASS4);
	    String classC1PropP3Range = Util.serializedRangeConstraint(Util.CLASS1, Util.PROP3, Util.CLASS5);	    
	    
	    /////////////////////// CLASS C2
	    ///////////// CLASS C2 SINGLETON + CWA
	    String classC2SingletonMin = Util.serializedSingletonMinConstraint(Util.CLASS2);
	    String classC2SingletonMax = Util.serializedSingletonMaxConstraint(Util.CLASS2);
	    String classC2CWA = Util.serializedPropCWA3Constraint(Util.CLASS2, Util.PROP4, Util.PROP5, Util.PROP6);
	    
	    ///////////// CLASS C2 PROPS
	    String classC2PropP4RangeType = Util.serializedRangeTypeConstraint(Util.CLASS2, Util.PROP4, "IRI", null);
	    String classC2PropP4Min = Util.serializedMin1Constraint(Util.CLASS2, Util.PROP4);
	    String classC2PropP4Max = Util.serializedMax3Constraint(Util.CLASS2, Util.PROP4);
	    String classC2PropP5RangeType = Util.serializedRangeTypeConstraint(Util.CLASS2, Util.PROP5, "BNODE", null);
	    String classC2PropP5Partiality = Util.serializedMax1Constraint(Util.CLASS2, Util.PROP5);
	    String classC2PropP6Min = Util.serializedMin1Constraint(Util.CLASS2, Util.PROP6);
	    String classC2PropP6Max = Util.serializedMax1Constraint(Util.CLASS2, Util.PROP6);	    

	    /////////////////////// CLASS C3
	    String classC3PropP7RangeType = Util.serializedRangeTypeConstraint(Util.CLASS3, Util.PROP7, "RESOURCE", null);
	    String classC3PropP7Path = Util.serializedPath1Constraint(Util.CLASS3, Util.PROP7, Util.PROP10);
	    String classC3PropP8SubProp1 = Util.serializedSubPropertyConstraint(Util.CLASS3, Util.PROP8, Util.PROP11);
	    String classC3PropP8SubProp2 = Util.serializedSubPropertyConstraint(Util.CLASS3, Util.PROP8, Util.PROP12);
	    String classC3PropP8SubProp3 = Util.serializedSubPropertyConstraint(Util.CLASS3, Util.PROP8, Util.PROP13);

	    /////////////////////// CLASS C4 (inherited)
	    String classC4PropP7RangeType = Util.serializedRangeTypeConstraint(Util.CLASS4, Util.PROP7, "RESOURCE", null);
	    String classC4PropP7Path = Util.serializedPath1Constraint(Util.CLASS4, Util.PROP7, Util.PROP10);
	    String classC4PropP8SubProp1 = Util.serializedSubPropertyConstraint(Util.CLASS4, Util.PROP8, Util.PROP11);
	    String classC4PropP8SubProp2 = Util.serializedSubPropertyConstraint(Util.CLASS4, Util.PROP8, Util.PROP12);
	    String classC4PropP8SubProp3 = Util.serializedSubPropertyConstraint(Util.CLASS4, Util.PROP8, Util.PROP13);

	    /*
	     * OWA PROPERTIES { 
	     *   example:p1 : LITERAL(<http://www.example.com/INT>);
	     *   DOMAIN(:C4) :p2 : LITERAL;
	     *   RANGE(<http://www.example.com/C5>) <http://www.example.com/p3>;
	     *   MIN(1), MAX(3) example:p4 : IRI;
	     *   PARTIAL example:p5 : BNODE;
	     *   TOTAL <http://www.example.com/p6>;
	     *   PATH(<http://www.example.com/p10>) <http://www.example.com/p7> : RESOURCE;
	     *   SUBPROPERTY(<http://www.example.com/p11>,example:p12,:p13) example:p8;
	     * }
		*/
//		Forall ?s1 ?o1 (IRI(?o1) :- ?s1[<http://www.example.com/p4> -> ?o1])
//		Forall ?s1 ?o1 (IRI(?o1) :- ?s1[<http://www.example.com/p4> -> ?o1])

	    String propP1RangeType = Util.serializedRangeTypeConstraint(null, Util.PROP1, "LITERAL", Util.INTEGER);
	    String propP2RangeType = Util.serializedRangeTypeConstraint(null, Util.PROP2, "LITERAL", null);
	    String propP2Domain = Util.serializedDomainConstraint(null, Util.PROP2, Util.CLASS4);
	    String propP3Range = Util.serializedRangeConstraint(null, Util.PROP3, Util.CLASS5);
	    String propP4RangeType = Util.serializedRangeTypeConstraint(null, Util.PROP4, "IRI", null);
	    String propP4Min = Util.serializedMin1Constraint(null, Util.PROP4);
	    String propP4Max = Util.serializedMax3Constraint(null, Util.PROP4);
	    String propP5RangeType = Util.serializedRangeTypeConstraint(null, Util.PROP5, "BNODE", null);
	    String propP5Partiality = Util.serializedMax1Constraint(null, Util.PROP5);
	    String propP6Min = Util.serializedMin1Constraint(null, Util.PROP6);
	    String propP6Max = Util.serializedMax1Constraint(null, Util.PROP6);	    
	    String propP7RangeType = Util.serializedRangeTypeConstraint(null, Util.PROP7, "RESOURCE", null);
	    String propP7Path = Util.serializedPath1Constraint(null, Util.PROP7, Util.PROP10);
	    String propP8SubProp1 = Util.serializedSubPropertyConstraint(null, Util.PROP8, Util.PROP11);
	    String propP8SubProp2 = Util.serializedSubPropertyConstraint(null, Util.PROP8, Util.PROP12);
	    String propP8SubProp3 = Util.serializedSubPropertyConstraint(null, Util.PROP8, Util.PROP13);

	    Util.assertSemantics(formulas,
	    	// CLASSES section (1)
	    	classes,
	    	// C1: Keys (13)
	    	classC1key1Min, classC1key1Max, classC1key1Key, classC1key1RangeType,
	    	classC1key2aMin, classC1key2bMin, classC1key2cMin, classC1key2aMax, classC1key2bMax, classC1key2cMax , classC1key2Key, classC1key2RangeType1, classC1key2RangeType2,
	    	// C1: Properties (4)
		    classC1PropP1RangeType, classC1PropP2RangeType, classC1PropP2Domain, classC1PropP3Range,
	    	// C2: Singleton + CWA (3)
		    classC2SingletonMin, classC2SingletonMax, classC2CWA,
		    // C2: Properties (7)
		    classC2PropP4RangeType, classC2PropP4Min, classC2PropP4Max, classC2PropP5RangeType, classC2PropP5Partiality, classC2PropP6Min, classC2PropP6Max,
		    // C3: Properties (5)
		    classC3PropP7RangeType, classC3PropP7Path, classC3PropP8SubProp1, classC3PropP8SubProp2, classC3PropP8SubProp3,
		    // C4: Properties (5)
		    classC4PropP7RangeType, classC4PropP7Path, classC4PropP8SubProp1, classC4PropP8SubProp2, classC4PropP8SubProp3,
		    // PROPERTIES (16)
		    propP1RangeType, propP2RangeType, propP2Domain, propP3Range, propP4RangeType, propP4Min, propP4Max, propP5RangeType, 
		    propP5Partiality, propP6Min, propP6Max, propP7RangeType, propP7Path, propP8SubProp1, propP8SubProp2, propP8SubProp3
	    );
	}
}