/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.semantics;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.Map;

import org.junit.Test;

import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.parser.ParseException;
import edu.freiburg.dbis.rdd.parser.RDDParser;
import edu.freiburg.dbis.rdd.sparql.IRI;

/**
 * Test different aspects of the environment extraction process.
 * 
 * @author MSchmidt
 */
public class EnvironmentExtractionTest {

	public static IRI CLASS_C1 = IRI.fromSparqlIri("<http://my.default.de/c1>");
	public static IRI CLASS_C2 = IRI.fromSparqlIri("<http://my.default.de/c2>");
	public static IRI CLASS_C3 = IRI.fromSparqlIri("<http://my.default.de/c3>");
	public static IRI CLASS_C4 = IRI.fromSparqlIri("<http://my.default.de/c4>");
	public static IRI CLASS_C5 = IRI.fromSparqlIri("<http://my.default.de/c5>");

	public static IRI PROP_P1 = IRI.fromSparqlIri("<http://www.example.com/p1>");
	public static IRI PROP_P2 = IRI.fromSparqlIri("<http://www.example.com/p2>");
	public static IRI PROP_P3 = IRI.fromSparqlIri("<http://www.example.com/p3>");
	public static IRI PROP_P4 = IRI.fromSparqlIri("<http://www.example.com/p4>");
	public static IRI PROP_P5 = IRI.fromSparqlIri("<http://www.example.com/p5>");
	public static IRI PROP_P6 = IRI.fromSparqlIri("<http://www.example.com/p6>");
	public static IRI PROP_P7 = IRI.fromSparqlIri("<http://www.example.com/p7>");
	public static IRI PROP_P8 = IRI.fromSparqlIri("<http://www.example.com/p8>");
	public static IRI PROP_P9 = IRI.fromSparqlIri("<http://www.example.com/p9>");
	public static IRI PROP_P10 = IRI.fromSparqlIri("<http://www.example.com/p10>");
	public static IRI PROP_P11 = IRI.fromSparqlIri("<http://www.example.com/p11>");
	public static IRI PROP_P12 = IRI.fromSparqlIri("<http://www.example.com/p12>");
	public static IRI PROP_P13 = IRI.fromSparqlIri("<http://www.example.com/p13>");
	public static IRI PROP_P14 = IRI.fromSparqlIri("<http://www.example.com/p14>");
	public static IRI PROP_P15 = IRI.fromSparqlIri("<http://www.example.com/p15>");
	public static IRI PROP_P16 = IRI.fromSparqlIri("<http://www.example.com/p16>");
	public static IRI PROP_P17 = IRI.fromSparqlIri("<http://www.example.com/p17>");
	public static IRI PROP_P18 = IRI.fromSparqlIri("<http://www.example.com/p18>");
	public static IRI PROP_P19 = IRI.fromSparqlIri("<http://www.example.com/p19>");
	public static IRI PROP_P20 = IRI.fromSparqlIri("<http://www.example.com/p20>");
	public static IRI PROP_P21 = IRI.fromSparqlIri("<http://www.example.com/p21>");
	public static IRI PROP_P22 = IRI.fromSparqlIri("<http://www.example.com/p22>");
	public static IRI PROP_P23 = IRI.fromSparqlIri("<http://www.example.com/p23>");
	public static IRI PROP_P24 = IRI.fromSparqlIri("<http://www.example.com/p24>");

	private static String TEST_BASE_DIR = "examples/test/environment/";

	@Test
	public void testEnvironmentExtraction()
	throws FileNotFoundException, ParseException {
		
		// STEP 1: first parsing round, and check that prefixes
		// are parsed correctly and prefix manager is properly
		// set up after parsing
		String rddFile = TEST_BASE_DIR + "Environment.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	
	    
	    // extract the environment
	    Environment env = new Environment(rdd);
	    
	    ///// STEP 1: verify C
	    Map<IRI,Collection<IRI>> envC = env.C;
	    assertEquals(3,envC.size());
	    
	    Collection<IRI> c1EnvC = envC.get(CLASS_C1);
	    assertTrue(c1EnvC.isEmpty());
	    
	    Collection<IRI> c2EnvC = envC.get(CLASS_C2);
	    assertEquals(1,c2EnvC.size());
	    assertTrue(c2EnvC.contains(CLASS_C3));
	    
	    Collection<IRI> c3EnvC = envC.get(CLASS_C3);
	    assertEquals(2,c3EnvC.size());
	    assertTrue(c3EnvC.contains(CLASS_C4));
	    assertTrue(c3EnvC.contains(CLASS_C5));
	    
	    ///// STEP 2: verify P
	    Map<IRI,Collection<IRI>> envP = env.P;
	    assertEquals(3,envP.size());

	    Collection<IRI> p11EnvP = envP.get(PROP_P11);
	    assertEquals(2,p11EnvP.size());
	    assertTrue(p11EnvP.contains(PROP_P12));
	    assertTrue(p11EnvP.contains(PROP_P13));
	    
	    Collection<IRI> p13EnvP = envP.get(PROP_P13);
	    assertEquals(2,p13EnvP.size());
	    assertTrue(p13EnvP.contains(PROP_P14));
	    assertTrue(p13EnvP.contains(PROP_P15));
	    
	    Collection<IRI> p23EnvP = envP.get(PROP_P23);
	    assertEquals(1,p23EnvP.size());
	    assertTrue(p23EnvP.contains(PROP_P24));
	      
	    ///// STEP 3: verify A
	    Map<IRI,Collection<IRI>> envA = env.A;
	    assertEquals(3,envA.size());

	    Collection<IRI> c1EnvA = envA.get(CLASS_C1);
	    assertEquals(11,c1EnvA.size());
	    assertTrue(c1EnvA.contains(PROP_P1));
	    assertTrue(c1EnvA.contains(PROP_P2));
	    assertTrue(c1EnvA.contains(PROP_P3));
	    assertTrue(c1EnvA.contains(PROP_P4));
	    assertTrue(c1EnvA.contains(PROP_P5));
	    assertTrue(c1EnvA.contains(PROP_P6));
	    assertTrue(c1EnvA.contains(PROP_P7));
	    assertTrue(c1EnvA.contains(PROP_P8));
	    assertTrue(c1EnvA.contains(PROP_P9));
	    assertTrue(c1EnvA.contains(PROP_P10));
	    assertTrue(c1EnvA.contains(PROP_P11));
	    
	    Collection<IRI> c2EnvA = envA.get(CLASS_C2);
	    assertEquals(2,c2EnvA.size());
	    assertTrue(c2EnvA.contains(PROP_P12));
	    assertTrue(c2EnvA.contains(PROP_P13));
	    
	    Collection<IRI> c3EnvA = envA.get(CLASS_C3);
	    assertEquals(2,c3EnvA.size());
	    assertTrue(c3EnvA.contains(PROP_P14));
	    assertTrue(c3EnvA.contains(PROP_P15));
	}
}
