/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.semantics;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Set;

import org.junit.Test;

import edu.freiburg.dbis.rdd.fol.FOFormula;
import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.parser.ParseException;
import edu.freiburg.dbis.rdd.parser.RDDParser;

/**
 * Test the implementation of the translation semantics of
 * the OWA, CWA and SINGLETON keyword.
 * 
 * @author MSchmidt
 */
public class MixedKeywordTest {

	protected static String TEST_BASE_DIR = "examples/test/semantics/";


	@Test
	public void testPropSecWorldAssumption() 
	throws FileNotFoundException, ParseException {
		
		String rddFileCWA = TEST_BASE_DIR + "PropSecClosedWorldAssumption.rdd";
		String rddFileOWA = TEST_BASE_DIR + "PropSecOpenWorldAssumption.rdd";
		FileInputStream fisCWA = new FileInputStream(new File(rddFileCWA));
		FileInputStream fisOWA = new FileInputStream(new File(rddFileOWA));

	    RDDParser parserCWA = new RDDParser(fisCWA);
	    RDDParser parserOWA = new RDDParser(fisOWA);
	    RDD rddCWA = parserCWA.rdd();
	    RDD rddOWA = parserOWA.rdd();

	    Set<FOFormula> formulasCWA = rddCWA.toFOL();
	    Set<FOFormula> formulasOWA = rddOWA.toFOL();

	    /**
	     * Two versions, where XWA is either OWA or CWA:
	     * 
	     * XWA PROPERTIES { 
	     *   MAX(1) <http://www.example.com/p1>;
	     *   MIN(1) <http://www.example.com/p2>;
	     *   PATH(<http://www.example.com/p4>) <http://www.example.com/p3>;
	     * }
	     */
	    String maxConstraint = 
	    	Util.serializedMax1Constraint(null, Util.PROP1);
	    String minConstraint = 
		    Util.serializedMin1Constraint(null, Util.PROP2);
	    String pathConstraint =
	    	Util.serializedPath1Constraint(null, Util.PROP3, Util.PROP4);

	    // Note: for some reason, in the result the order of properties in the
	    // or clause is p2, p1, p3, so we have to adjust the formula. In case
	    // this fails, we may need to adjust the order.
	    String cwaConstraint = 
		    Util.serializedPropCWA3Constraint(null, Util.PROP2, Util.PROP1, Util.PROP3);
	    
	    // under CWA, a respective closed world constraint is derived
	    Util.assertSemantics(formulasCWA,
	    	maxConstraint, minConstraint, pathConstraint, cwaConstraint);

	    // under OWA, no closed world constraint is derived
	    Util.assertSemantics(formulasOWA,
		    	maxConstraint, minConstraint, pathConstraint);
	}

	@Test
	public void testClassesSecWorldAssumption() 
	throws FileNotFoundException, ParseException {
		
		String rddFileCWA = TEST_BASE_DIR + "ClassesSecClosedWorldAssumption.rdd";
		String rddFileOWA = TEST_BASE_DIR + "ClassesSecOpenWorldAssumption.rdd";
		FileInputStream fisCWA = new FileInputStream(new File(rddFileCWA));
		FileInputStream fisOWA = new FileInputStream(new File(rddFileOWA));

	    RDDParser parserCWA = new RDDParser(fisCWA);
	    RDDParser parserOWA = new RDDParser(fisOWA);
	    RDD rddCWA = parserCWA.rdd();	
	    RDD rddOWA = parserOWA.rdd();	

	    Set<FOFormula> formulasCWA = rddCWA.toFOL();
	    Set<FOFormula> formulasOWA = rddOWA.toFOL();

	    /**
	     * Two versions, where XWA is either OWA or CWA:
	     * 
	     * XWA CLASSES {
	     *   OWA CLASS <http://www.example.com/C1> {}
	     *   OWA CLASS <http://www.example.com/C2> {}
	     *   OWA CLASS <http://www.example.com/C3> {}  
	     * }
	     */
	    // Note: for some reason, in the result the order of classes in the
	    // or clause is C1, C3, C2, so we have to adjust the formula. In case
	    // this fails, we may need to adjust the order.
	    String cwaConstraint = 
	    	Util.serializedClassesCWA3Constraint(Util.CLASS1, Util.CLASS3, Util.CLASS2); 
	    
	    Util.assertSemantics(formulasCWA, cwaConstraint);
	    Util.assertSemantics(formulasOWA);
	}
	
	@Test
	public void testClassSecWorldAssumption() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "ClassSecWorldAssumption.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * Class C1 is defined as follows.
	     * 
	     * CWA CLASS <http://www.example.com/C1> { 
	     *   MAX(1) <http://www.example.com/p1>;
	     *   MIN(1) <http://www.example.com/p2>;
	     *   PATH(<http://www.example.com/p4>) <http://www.example.com/p3>;
	     * }
	     * 
	     * Class C2 is the same modulo keyword OWA (instead of CWA).
	     */
	    String maxConstraintC1 =
		    Util.serializedMax1Constraint(Util.CLASS1, Util.PROP1);
	    String minConstraintC1 = 
	    	Util.serializedMin1Constraint(Util.CLASS1, Util.PROP2);
	    String pathConstraintC1 =
		    Util.serializedPath1Constraint(Util.CLASS1, Util.PROP3, Util.PROP4);
	    String cwaConstraintC1 = 
	    	Util.serializedPropCWA3Constraint(Util.CLASS1, Util.PROP2, Util.PROP1, Util.PROP3);
	    
	    String maxConstraintC2 =
	    	Util.serializedMax1Constraint(Util.CLASS2, Util.PROP1);
		String minConstraintC2 = 
		    Util.serializedMin1Constraint(Util.CLASS2, Util.PROP2);
		String pathConstraintC2 =
			Util.serializedPath1Constraint(Util.CLASS2, Util.PROP3, Util.PROP4);
	    
	    Util.assertSemantics(formulas,
	    	minConstraintC1, maxConstraintC1, pathConstraintC1, cwaConstraintC1, 
	    	minConstraintC2, maxConstraintC2, pathConstraintC2);
	}

	/**
	 * Test (i) CWA constraint with empty prop sec and
	 * (ii) CWA with empty classes section
	 */
	@Test
	public void testWorldAssumptionCWAEmptyClassesEmptyProperties()
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "WorldAssumptionCWAEmptyClassesEmptyProperties.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * CWA CLASSES {}
	     * CWA PROPERTIES {}
	     */
	    String classesCWAConstraint = 
	    	"Forall ?s1 ?o1("
		    + "FALSE :- ?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> ?o1]"
		    + ")";	    
	    String propertiesCWAConstraint =
	    	"Forall ?s1 ?p1 ?o1("
    		+ "Or(?p1=<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>) :- ?s1[?p1 -> ?o1]"
    		+ ")";

	    Util.assertSemantics(formulas,
	    	classesCWAConstraint, propertiesCWAConstraint);
	}	
	
	/**
	 * Test CWA constraint for class having no properties
	 */
	@Test
	public void testWorldAssumptionCWAEmptyClass()
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "WorldAssumptionCWAEmptyClass.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASSES {
	     *   CWA CLASS <http://www.example.com/C1> {}
	     * }
	     */
	    String constraint =
		    "Forall ?s1 ?p1 ?o1("
		    + "Or(?p1=<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>) "
		    + ":- And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> <http://www.example.com/C1>] "
		    + "       ?s1[?p1 -> ?o1])"
		    + ")";

	    Util.assertSemantics(formulas, constraint);
	}	

	
	@Test
	public void testSingletonClass() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "SingletonClass.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASSES {
	     *   OWA CLASS <http://www.example.com/C1> {}
	     *   OWA SINGLETON CLASS <http://www.example.com/C2> {}
	     * }
	     */
	    String singletonMaxConstraint = Util.serializedSingletonMaxConstraint(Util.CLASS2);
	    String singletonMinConstraint = Util.serializedSingletonMinConstraint(Util.CLASS2);

	    Util.assertSemantics(formulas, singletonMinConstraint, singletonMaxConstraint);
	}
}