/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.semantics;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Set;

import org.junit.Test;

import edu.freiburg.dbis.rdd.fol.FOFormula;
import edu.freiburg.dbis.rdd.model.RDD;
import edu.freiburg.dbis.rdd.parser.ParseException;
import edu.freiburg.dbis.rdd.parser.RDDParser;

/**
 * Test the implementation of the subclassing hierarchy semantics,
 * i.e. proper inheritance of constraints along implied class hierarchies.
 * 
 * @author MSchmidt
 */
public class SubclassTest {

	protected static String TEST_BASE_DIR = "examples/test/semantics/";

	@Test
	public void testSubclassInheritance() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "SubclassInheritance.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASSES {
	     *   CWA SINGLETON CLASS <http://www.example.com/C1> SUBCLASS <http://www.example.com/C2> {
	     *     KEY <http://www.example.com/p1>;
	     *     MIN(1) <http://www.example.com/p2> : IRI;
	     *     PARTIALITY <http://www.example.com/p3>;
	     *   }
	     *
	     *   OWA CLASS <http://www.example.com/C2> SUBCLASS <http://www.example.com/C3>, <http://www.example.com/C4> {
	     *     KEY <http://www.example.com/p4>;
	     *     MAX(1) <http://www.example.com/p5>;
	     *   }
	     * }
	     */
	    // CLASS C1
	    String class1SingletonMin =
	    	Util.serializedSingletonMinConstraint(Util.CLASS1);
	    String class1SingletonMax =
		    	Util.serializedSingletonMaxConstraint(Util.CLASS1);
	    String class1CWA =
		    	Util.serializedPropCWA3Constraint(Util.CLASS1, Util.PROP2, Util.PROP1, Util.PROP3);
	    String class1KeyMin1 =
	    	Util.serializedMin1Constraint(Util.CLASS1, Util.PROP1);
	    String class1KeyMax1 =
		    Util.serializedMax1Constraint(Util.CLASS1, Util.PROP1);
	    String class1KeyKey =
		    Util.serializedKey1Constraint(Util.CLASS1, Util.PROP1);
	    String class1Min =
	    	Util.serializedMin1Constraint(Util.CLASS1, Util.PROP2);
	    String class1RangeType =
		    Util.serializedRangeTypeConstraint(Util.CLASS1, Util.PROP2, "IRI", null);
	    String class1Partiality =
		    Util.serializedMax1Constraint(Util.CLASS1, Util.PROP3);

	    // CLASS C2 inherited from C1
	    String inheritedClass2KeyMin1 =
	    	Util.serializedMin1Constraint(Util.CLASS2, Util.PROP1);
	    String inheritedClass2KeyMax1 =
	    	Util.serializedMax1Constraint(Util.CLASS2, Util.PROP1);
	    String inheritedClass2KeyKey =
	    	Util.serializedKey1Constraint(Util.CLASS2, Util.PROP1);
	    String inheritedClass2Min =
	    	Util.serializedMin1Constraint(Util.CLASS2, Util.PROP2);
	    String inheritedClass2RangeType =
			Util.serializedRangeTypeConstraint(Util.CLASS2, Util.PROP2, "IRI", null);
	    String inheritedClass2Partiality =
	    	Util.serializedMax1Constraint(Util.CLASS2, Util.PROP3);

	    // CLASS C2
	    String class2KeyMin1 =
		    Util.serializedMin1Constraint(Util.CLASS2, Util.PROP4);
		String class2KeyMax1 =
			Util.serializedMax1Constraint(Util.CLASS2, Util.PROP4);
		String class2KeyKey =
			Util.serializedKey1Constraint(Util.CLASS2, Util.PROP4);
	    String class2Max=
	    	Util.serializedMax1Constraint(Util.CLASS2, Util.PROP5);

	    // CLASS C3 inherited from C1
	    String inheritedClass3KeyMin1 =
		    Util.serializedMin1Constraint(Util.CLASS3, Util.PROP1);
	    String inheritedClass3KeyMax1 =
	    	Util.serializedMax1Constraint(Util.CLASS3, Util.PROP1);
	    String inheritedClass3KeyKey =
	    	Util.serializedKey1Constraint(Util.CLASS3, Util.PROP1);
	    String inheritedClass3Min =
	    	Util.serializedMin1Constraint(Util.CLASS3, Util.PROP2);
	    String inheritedClass3RangeType =
	    	Util.serializedRangeTypeConstraint(Util.CLASS3, Util.PROP2, "IRI", null);
	    String inheritedClass3Partiality =
	    	Util.serializedMax1Constraint(Util.CLASS3, Util.PROP3);
	    
	    // CLASS C3 inherited from C2
	    String class3KeyMin1 =
			Util.serializedMin1Constraint(Util.CLASS3, Util.PROP4);
	    String class3KeyMax1 =
	    	Util.serializedMax1Constraint(Util.CLASS3, Util.PROP4);
	    String class3KeyKey =
	    	Util.serializedKey1Constraint(Util.CLASS3, Util.PROP4);
	    String class3Max=
	    	Util.serializedMax1Constraint(Util.CLASS3, Util.PROP5);

	    // CLASS C4 inherited from C1
	    String inheritedClass4KeyMin1 =
		    Util.serializedMin1Constraint(Util.CLASS4, Util.PROP1);
	    String inheritedClass4KeyMax1 =
	    	Util.serializedMax1Constraint(Util.CLASS4, Util.PROP1);
	    String inheritedClass4KeyKey =
	    	Util.serializedKey1Constraint(Util.CLASS4, Util.PROP1);
	    String inheritedClass4Min =
	    	Util.serializedMin1Constraint(Util.CLASS4, Util.PROP2);
	    String inheritedClass4RangeType =
	    	Util.serializedRangeTypeConstraint(Util.CLASS4, Util.PROP2, "IRI", null);
	    String inheritedClass4Partiality =
	    	Util.serializedMax1Constraint(Util.CLASS4, Util.PROP3);
	    
	    // CLASS C4
	    String class4KeyMin1 =
			Util.serializedMin1Constraint(Util.CLASS4, Util.PROP4);
	    String class4KeyMax1 =
	    	Util.serializedMax1Constraint(Util.CLASS4, Util.PROP4);
	    String class4KeyKey =
	    	Util.serializedKey1Constraint(Util.CLASS4, Util.PROP4);
	    String class4Max=
	    	Util.serializedMax1Constraint(Util.CLASS4, Util.PROP5);
	    
	    Util.assertSemantics(formulas,
	    	// CLASS 1 constraints
	    	class1SingletonMax, class1SingletonMin, class1CWA, class1KeyMin1, class1KeyMax1, class1KeyKey,
		    class1Min, class1RangeType, class1Partiality,
		    // CLASS C2 inherited from C1
		    inheritedClass2KeyMin1, inheritedClass2KeyMax1, inheritedClass2KeyKey, inheritedClass2Min, inheritedClass2RangeType, inheritedClass2Partiality,
		    // CLASS C2
		    class2KeyMin1, class2KeyMax1, class2KeyKey, class2Max,
		    // CLASS C3 inherited from C1
		    inheritedClass3KeyMin1, inheritedClass3KeyMax1, inheritedClass3KeyKey, inheritedClass3Min, inheritedClass3RangeType, inheritedClass3Partiality,
		    // CLASS C3 inherited from C2
		    class3KeyMin1, class3KeyMax1, class3KeyKey, class3Max,
		    // CLASS C4 inherited from C1
		    inheritedClass4KeyMin1, inheritedClass4KeyMax1, inheritedClass4KeyKey, inheritedClass4Min, inheritedClass4RangeType, inheritedClass4Partiality,
		    // CLASS C4 inherited from C2
		    class4KeyMin1, class4KeyMax1, class4KeyKey, class4Max
		    );
	}

	@Test
	public void testSubclassInheritanceWithCycle() 
	throws FileNotFoundException, ParseException {
		
		String rddFile = TEST_BASE_DIR + "SubclassInheritanceWithCycle.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASSES {
	     *   OWA CLASS <http://www.example.com/C1> SUBCLASS <http://www.example.com/C2> {
	     *     MIN(1) <http://www.example.com/p1>;
	     *   }
	     *   
	     *   OWA CLASS <http://www.example.com/C2> SUBCLASS <http://www.example.com/C3>, <http://www.example.com/C4> {
	     *     MAX(1) <http://www.example.com/p2>;
	     *   }
	     *   
	     *   OWA CLASS <http://www.example.com/C4> SUBCLASS <http://www.example.com/C1> {
	     *     DOMAIN(<http://www.example.com/C5>) <http://www.example.com/p3>;
	     *   }
	     * }
	     */
	    String minClass1 = Util.serializedMin1Constraint(Util.CLASS1, Util.PROP1);
	    String minClass2 = Util.serializedMin1Constraint(Util.CLASS2, Util.PROP1);
	    String minClass3 = Util.serializedMin1Constraint(Util.CLASS3, Util.PROP1);
	    String minClass4 = Util.serializedMin1Constraint(Util.CLASS4, Util.PROP1);

	    String maxClass1 = Util.serializedMax1Constraint(Util.CLASS1, Util.PROP2);
	    String maxClass2 = Util.serializedMax1Constraint(Util.CLASS2, Util.PROP2);
	    String maxClass3 = Util.serializedMax1Constraint(Util.CLASS3, Util.PROP2);
	    String maxClass4 = Util.serializedMax1Constraint(Util.CLASS4, Util.PROP2);

	    String domainClass1 = Util.serializedDomainConstraint(Util.CLASS1, Util.PROP3, Util.CLASS5);
	    String domainClass2 = Util.serializedDomainConstraint(Util.CLASS2, Util.PROP3, Util.CLASS5);
	    String domainClass3 = Util.serializedDomainConstraint(Util.CLASS3, Util.PROP3, Util.CLASS5);
	    String domainClass4 = Util.serializedDomainConstraint(Util.CLASS4, Util.PROP3, Util.CLASS5);

	    Util.assertSemantics(formulas,
	    	minClass1, minClass2, minClass3, minClass4,
	    	maxClass1, maxClass2, maxClass3, maxClass4,
	    	domainClass1, domainClass2, domainClass3, domainClass4);
	}
	
	@Test
	public void testSubclassSelfCycle() 
	throws FileNotFoundException, ParseException {
		String rddFile = TEST_BASE_DIR + "SubclassSelfCycle.rdd";
		FileInputStream fis = new FileInputStream(new File(rddFile));

	    RDDParser parser = new RDDParser(fis);
	    RDD rdd = parser.rdd();	

	    Set<FOFormula> formulas = rdd.toFOL();

	    /**
	     * OWA CLASSES {
	     *   OWA CLASS <http://www.example.com/C1> SUBCLASS <http://www.example.com/C1> {
	     *     MIN(1) <http://www.example.com/p1>;
	     *   }
	     * }
	     */
	    String minClass1 = Util.serializedMin1Constraint(Util.CLASS1, Util.PROP1);
	    
	    Util.assertSemantics(formulas, minClass1);
	}
}