/*
Copyright 2014, Albert-Ludwigs-Universitt Freiburg (Database Group)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package edu.freiburg.dbis.rdd.semantics;

import static org.junit.Assert.assertEquals;

import java.util.Set;

import edu.freiburg.dbis.rdd.fol.FOFormula;

/**
 * Semantics test utility used throughout different tests.
 * 
 * @author MSchmidt
 */
public class Util {

	public static String RDF_TYPE = "<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>";

	public static String CLASS1 = "<http://www.example.com/C1>";
	public static String CLASS2 = "<http://www.example.com/C2>";
	public static String CLASS3 = "<http://www.example.com/C3>";
	public static String CLASS4 = "<http://www.example.com/C4>";
	public static String CLASS5 = "<http://www.example.com/C5>";

	public static String PROP1 = "<http://www.example.com/p1>";
	public static String PROP2 = "<http://www.example.com/p2>";
	public static String PROP3 = "<http://www.example.com/p3>";
	public static String PROP4 = "<http://www.example.com/p4>";
	public static String PROP5 = "<http://www.example.com/p5>";
	public static String PROP6 = "<http://www.example.com/p6>";
	public static String PROP7 = "<http://www.example.com/p7>";
	public static String PROP8 = "<http://www.example.com/p8>";
	public static String PROP9 = "<http://www.example.com/p9>";
	public static String PROP10 = "<http://www.example.com/p10>";
	public static String PROP11 = "<http://www.example.com/p11>";
	public static String PROP12 = "<http://www.example.com/p12>";
	public static String PROP13 = "<http://www.example.com/p13>";
	public static String PROP14 = "<http://www.example.com/p14>";
	public static String PROP15 = "<http://www.example.com/p15>";

	public static String INTEGER = "<http://www.example.com/INT>";
	
	/**
	 * Asserts that the serialization of the passes formula list equals
	 * (in any order) to the passed strings. Throws a {@link RuntimeException}
	 * to indicate assertion failure.
	 */
	public static void assertSemantics(Set<FOFormula> formulas, String... constraints) {

		Set<FOFormula> minimizedFormulas = FOFormula.minimizeByProvenanceRemoval(formulas);
		
		assertEquals(constraints.length, minimizedFormulas.size());
		
		int fi=0;
		for (FOFormula formula : minimizedFormulas) {
			System.err.println("\n### Matching nr " + ++fi + ":\n" + formula.toRIF());
			boolean matched = false;
			for (int i=0; i<constraints.length && !matched; i++) {
				System.out.println(constraints[i]);
				matched = formulaMatchesConstraint(formula.toRIF(),constraints[i]);
				if (matched)
					constraints[i] = "MATCHED"; // mark as matched, will not match again
			}
			
			if (!matched) {
				String msg = "The following formula was extracted, but not expected:\n" + formula.toRIF();
				throw new RuntimeException(msg);
			}
		}
	}
	
	/**
	 * String equivalence check ignoring white spaces.
	 */
	public static boolean formulaMatchesConstraint(String formula, String constraint) {
		return formula.replaceAll(" ", "").equals(constraint.replaceAll(" ", ""));
	}

	/**
	 * Serialized version of RANGE(1) constraint.
	 */
	public static String serializedRangeConstraint(String clazz, String prop, String range) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?o1 ");
		buf.append("(?o1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + range + "] :- ");
		if (clazz!=null) {
			buf.append("And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
			buf.append("?s1[" + prop + " -> ?o1]))");
		} else {
			buf.append("?s1[" + prop + " -> ?o1])");
		}
		
		return buf.toString();
	}

	/**
	 * Serialized version of DOMAIN(1) constraint.
	 */
	public static String serializedDomainConstraint(String clazz, String prop, String domain) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?o1 ");
		buf.append("(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + domain + "] :- ");
		if (clazz!=null) {
			buf.append("And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
			buf.append("?s1[" + prop + " -> ?o1]))");
		} else {
			buf.append("?s1[" + prop + " -> ?o1])");
		}
		
		return buf.toString();
	}
	
	/**
	 * Serialized version of MIN(1) constraint.
	 */
	public static String serializedMin1Constraint(String clazz, String prop) {
	    StringBuffer buf = new StringBuffer();
	    buf.append("Forall ?s1 ");
    	buf.append("(Exists ?o1 (?s1[" + prop + " -> ?o1]) :- ");
	    if (clazz!=null)
			buf.append("?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "])");
	    else
		    buf.append("RESOURCE(?s1))");
	    
	    return buf.toString();		
	}

	/**
	 * Serialized version of MIN(3) constraint.
	 */
	public static String serializedMin3Constraint(String clazz, String prop) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ");
		buf.append("(Exists ?o1 ?o2 ?o3 (");
		buf.append("  And(");
		buf.append("    ?s1[" + prop + " -> ?o1] ");
		buf.append("    ?s1[" + prop + " -> ?o2] ");
		buf.append("    ?s1[" + prop + " -> ?o3]");
		buf.append("    Not(?o1=?o2)");
		buf.append("    Not(?o1=?o3)");
		buf.append("    Not(?o2=?o3))) ");
		if (clazz!=null)
			buf.append(":- ?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "])");
		else
			buf.append(":- RESOURCE(?s1))");
		
		return buf.toString();
	}
	
	/**
	 * Serialized version of MAX(1) constraint.
	 */
	public static String serializedMax1Constraint(String clazz, String prop) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?o1 ?o2");
		buf.append("(?o1=?o2 :-");
		buf.append(" And(");
		if (clazz!=null)
			buf.append("?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
		buf.append("     ?s1[" + prop + " -> ?o1] ");
		buf.append("     ?s1[" + prop + " -> ?o2]))");
		
		return buf.toString();
	}
	
	/**
	 * Serialized version of MAX(3) constraint.
	 */
	public static String serializedMax3Constraint(String clazz, String prop) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?o1 ?o2 ?o3 ?o4");
		buf.append("(Or(?o1=?o2 ?o1=?o3 ?o1=?o4 ?o2=?o3 ?o2=?o4 ?o3=?o4) :-");
		if (clazz!=null) {
			buf.append(" And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
			buf.append("     ?s1[" + prop + " -> ?o1] ");
		} else {
			buf.append(" And(?s1[" + prop + " -> ?o1] ");
		}
		buf.append("     ?s1[" + prop + " -> ?o2] ");
		buf.append("     ?s1[" + prop + " -> ?o3] ");
		buf.append("     ?s1[" + prop + " -> ?o4]))");

		return buf.toString();
	}
	
	/**
	 * Serialized version of path of length 1 constraint.
	 */
	public static String serializedPath1Constraint(String clazz, String prop, String pathProp) {
	    StringBuffer buf = new StringBuffer();
	    buf.append("Forall ?s1 ?o1 ");
		buf.append("(?s1[" + pathProp + " -> ?o1] :-");
		if (clazz!=null) {
			buf.append(" And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "]");
			buf.append("     ?s1[" + prop + " -> ?o1]))");
		} else {
			buf.append(" ?s1[" + prop + " -> ?o1])");
		}
		
	    return buf.toString();
	}
	
	/**
	 * Serialized version of path of length 3 constraint.
	 */
	public static String serializedPath3Constraint(
			String clazz, String prop, 
			String pathProp1, String pathProp2, String pathProp3) {
		
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?o1 ");
		buf.append("(Exists ?o2 ?o3(");
		buf.append(" And(?s1[" + pathProp1 + " -> ?o2] ");
		buf.append("     ?o2[" + pathProp2 + " -> ?o3] ");
		buf.append("     ?o3[" + pathProp3 + " -> ?o1])) :-");
		if (clazz!=null) {
			buf.append(" And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
			buf.append("     ?s1[" + prop + " -> ?o1]))");
		} else {
			buf.append(" ?s1[" + prop + " -> ?o1])");
		}

		return buf.toString();
	}

	/**
	 * Serialized version of path of subproperty constraints.
	 */
	public static String serializedSubPropertyConstraint(String clazz, String prop, String subProp) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?o1 ");
		buf.append("(?s1[" + prop + " -> ?o1] :-");
		if (clazz!=null) {
			buf.append("And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "]");
			buf.append("     ?s1[" + subProp + " -> ?o1]))");
		} else {
			buf.append(" ?s1[" + subProp + " -> ?o1])");
		}
		
		return buf.toString();
	}
	
	/**
	 * Serialized version of range type constraint.
	 */
	public static String serializedRangeTypeConstraint(String clazz, String prop, String relation, String datatype) {
	    StringBuffer buf = new StringBuffer();
	    buf.append("Forall ?s1 ?o1");
	    if (datatype!=null) {
	    	buf.append("(And(" + relation + "(?o1) ");
	    	buf.append("    DATATYPE(?o1," + datatype + ")) :- ");
	    } else {
	    	buf.append("(" + relation + "(?o1) :- ");
	    }
	    
	    if (clazz!=null) {
    		buf.append("And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "]");
    		buf.append("    ?s1[" + prop + " -> ?o1]))");
	    } else {
    		buf.append("    ?s1[" + prop + " -> ?o1])");	    	
	    }
	    
	    return buf.toString();
	}

	/**
	 * Serialized version of singleton constraint max part.
	 */
	public static String serializedSingletonMaxConstraint(String clazz) {
	    StringBuffer buf = new StringBuffer();
	    buf.append("Forall ?s1 ?s2(");
	    buf.append("?s1=?s2 :- ");
	    buf.append("And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
	    buf.append("    ?s2[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "]))");
	    
	    return buf.toString();
	}

	/**
	 * Serialized version of singleton constraint min part.
	 */
	public static String serializedSingletonMinConstraint(String clazz) {
	    StringBuffer buf = new StringBuffer();
	    buf.append("Exists ?s1 (?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "])");
	    
	    return buf.toString();
	}

	
	/**
	 * Serialized version of CLASS CWA or PROPERTIES CWA constraint for three properties.
	 */
	public static String serializedPropCWA3Constraint(String clazz, String prop1, String prop2, String prop3) {
	    StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?p1 ?o1(");
		buf.append("Or(?p1=" + RDF_TYPE + " ");
	    buf.append("   ?p1=" + prop1 + " ");
	    buf.append("   ?p1=" + prop2 + " ");
	    buf.append("   ?p1=" + prop3 + ")");
	    if (clazz!=null) {
	    	buf.append(":- And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
	    	buf.append("       ?s1[?p1 -> ?o1]))");
	    } else {
			buf.append(":- ?s1[?p1 -> ?o1])");	    	
	    }
	    
	    return buf.toString();
	}
	
	/**
	 * Serialized version of CLASSES CWA constraint for 3 classes. 
	 */
	public static String serializedClassesCWA3Constraint(String clazz1, String clazz2, String clazz3) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?o1(");
		buf.append("Or(?o1=" + clazz1 + " ");
		buf.append("   ?o1=" + clazz2 + " ");
		buf.append("   ?o1=" + clazz3 + ")");
		buf.append(":- ?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> ?o1])");
		
		return buf.toString();
	}
	
	public static String serializedKey1Constraint(String clazz, String keyProp) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?s2 ?o1(");
		buf.append("?s1=?s2 :- ");
		buf.append("And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
    	buf.append("    ?s1[" + keyProp + " -> ?o1] ");
    	buf.append("    ?s2[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
    	buf.append("    ?s2[" + keyProp + " -> ?o1]))");

    	return buf.toString();
	}
	
	public static String serializedKey3Constraint(String clazz, 
			String keyProp1, String keyProp2, String keyProp3) {
		StringBuffer buf = new StringBuffer();
		buf.append("Forall ?s1 ?s2 ?o1 ?o2 ?o3(");
		buf.append("?s1=?s2 :- ");
		buf.append("And(?s1[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
		buf.append("    ?s1[" + keyProp1 + " -> ?o1] ");
		buf.append("    ?s1[" + keyProp2 + " -> ?o2] ");
		buf.append("    ?s1[" + keyProp3 + " -> ?o3] ");
		buf.append("    ?s2[<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> -> " + clazz + "] ");
		buf.append("    ?s2[" + keyProp1 + " -> ?o1] ");
		buf.append("    ?s2[" + keyProp2 + " -> ?o2] ");
		buf.append("    ?s2[" + keyProp3 + " -> ?o3]))");

		return buf.toString();
	}		
}
